/*
 * MAX : scanmate.cc, Tue Jan  8 16:27:10 CET 2002 -sg
 *
 * Device driver for the Scanmate Dye Laser
 * 	Serial Device is handled within this Driver because no other 
 *	device needs this (by now)
 *
 * 02.01.2002: scanmate-driver still alpha (needs testing !)
 * 		-> '\r' ends commands to scanmate !
 * 		-> no CRTCTS !
 * 		-> scan via grating and crystal OK
 * 08.01.2002: while-loop in set_grating and set_crystal improved
			(count<10, abs(...)>1)
 *
 * This file is part of Max data acquisition software
 * (c) 2001 by S. Gerlach
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include "list.hh"
#include "sched.hh"
#include "camacs.h"
#include "error.hh"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>

#define BAUDRATE B9600            
#define SERIALDEVICE "/dev/cua1"	// for Kernel 2.0 (>=2.2 : /dev/ttyS1)

#define DEBUG_DEV_SCANMATE

// global variables
static int sfd;			// serial fd 
struct termios oldtio, newtio;	// old + new terminal settings
char buf[255], msg[255];	// Serial reading buffer + message
static double wavelength;	// actual wavelength
static double start, stop;	// start, stop wavelength

/*****************************************************************************/
int get_status() {
	int n, status = 100;
	
	sprintf(msg,"\rD:OPMODE?\r");
	write(sfd,msg,strlen(msg)); 

	//memset(ans,0,sizeof(ans));
	read(sfd,buf,255);


#ifdef DEBUG_DEV_SCANMATE
        buf[n]=0;
        printf("SCANMATE (D:OPMODE?) :%s\n", buf);
#endif

	// TODO
	/* if (findstr(1,ans,"OFF:") > 0)  status = atoi(ans+4);

	if (findstr(1,ans,"SET,") > 0) {
		status = 10;
		if (findstr(4,ans,"GRATING") > 0) status += 1;
		if (findstr(4,ans,"CRYSTAL") > 0) status += 2;
		if (findstr(4,ans,"ETALON") > 0) status += 3;
	}
	if (findstr(1,ans,"SCAN,") > 0) {
		status = 20;
		if (findstr(4,ans,"BURST") > 0) status += 1;
		if (findstr(4,ans,"CRYSTAL") > 0) status += 2;
		if (findstr(4,ans,"ETALON") > 0) status += 3;
	}*/
	  
	return(status);
}

double get_wavelength() {
	int n;
	double l;

	sprintf(msg,"\rD:WAVELENGTH?\r");
	write(sfd,msg,strlen(msg)); 
	memset(buf,0,255);   		 
	n = read(sfd,buf,255);

#ifdef DEBUG_DEV_SCANMATE
        buf[n]=0;
        printf("SCANMATE (D:WAVELENGTH?) :%s", buf);
#endif
	
	l = atof(buf);
	
	return(l);
}

int set_wavelength_grating(double l) {
	
	/* goto wavelength */
	sprintf(msg,"\rD:OPMODE=SET,GRATING(%.3f)\r",l);

	write(sfd,msg,strlen(msg)); 

#ifdef DEBUG_DEV_SCANMATE
        printf("SCANMATE (D:OPMODE=SET,GRATING(%.3f))\n",l);
#endif

	int count=0;
	while (abs((int)((l-2*get_wavelength())*1000))>1 && count < 10) {
		usleep(300000); // waiting for grating
		count++;
	}
	
	return 1;
}	


int set_wavelength_crystal(double l) {

	/* goto wavelength */
	sprintf(msg,"\rD:OPMODE=SET,CRYSTAL(%.3f)\r",l);

	write(sfd,msg,strlen(msg)); 

#ifdef DEBUG_DEV_SCANMATE
        printf("SCANMATE (D:OPMODE=SET,CRYSTAL(%.3f))\n",l);
#endif

	int count=0;
	while (abs((int)((l-2*get_wavelength())*1000))>1 && count < 10) {
		usleep(300000); // waiting for grating
		count++;
	}
	
	/* get answer */
	/*if (((status = get_status()) == 1) || (status == 2)) {
	  	printf("Scanmate (%s) : not accepted from dyelaser (%d)\n",
				msg,status);
		return(0);
	}*/

	/* wait for position */
	/*while (abs((int)((l-2*get_wavelength())*1000))>1) {
		// sleep(1);
	}*/

	return(1);
}

int crystal_calibrated() {
	int n;
	
	/* get calibration data */
	sprintf(msg,"\rD:CAL CRYSTAL?\r");
	write(sfd,msg,strlen(msg)); 
	
	//memset(buf,0,sizeof(buf));	 
	n = read(sfd,buf,255);

#ifdef DEBUG_DEV_SCANMATE
        buf[n]=0;
        printf("SCANMATE (D:CAL CRYSTAL?) :%s:%d\n", buf, n);
#endif
	
	if (strcmp(buf,"CALIBRATE") ==  0)
		return(1);
	else 
		return(0);
} 

int wavelength_valid(double l) {
	int n, pos1, pos2;
	char crystal[20];
	//int pos1, pos2;
	//double init1, init2, init_min, init_max;
	//char initstr1[20],initstr2[20];

	/* get crystal calibration data*/
	sprintf(msg,"\rD:CAL CRYSTAL?\r");
	write(sfd,msg,strlen(msg)); 
	
	//memset(buf,0,sizeof(buf));   		 
	read(sfd,buf,255);

#ifdef DEBUG_DEV_SCANMATE
        buf[n]=0;
        printf("SCANMATE (D:CAL CRYSTAL?) :%s:%d\n", buf, n);
#endif
	
	if (strcmp(buf,"CALIBRATE") ==  0) {	// not calibrated
		printf("Scanmate : Crystal not calibrated !\n");
		return(0);
	}

	sscanf(buf,"(%s,%d,%d)",crystal, &pos1, &pos2);

#ifdef DEBUG_DEV_SCANMATE
        printf("Scanmate :  Crystal :%s [Start = %d, Stop = %d]\n",crystal,pos1,pos2);
#endif
	
	// TODO
	/* pos1 = findstr(1,ans,",",strlen(ans));
	pos2 = findstr(pos1+1,ans,",",strlen(ans)-pos1);
	if (pos1*pos2 != 0) {
		memset(initstr1,0,sizeof(initstr1));   		 
		strncat(initstr1,r+pos1,pos2-pos1-1);
		init1 = atof(initstr1);
		memset(initstr2,0,sizeof(initstr2));   		 
		strncat(initstr2,r+pos2,strlen(r)-pos2-2);
		init2 = atof(initstr2);
		init_max = (init1>init2) ? init1 : init2;
		init_min = (init1<init2) ? init1 : init2;
	}
	else
	  	return(0);
	*/

	/* wavelength valid ?? */
	// l *= 2;
	/*if ((l<init_min) || (l>init_max)) 
	  	return (0);
	else */
	
	  return (1);
} 

// see Serial-Programming-HOWTO
int init_serial() {
	cout <<"Opening serial device\n";
	sfd = open(SERIALDEVICE, O_RDWR | O_NOCTTY ); 
	if (sfd <0) {
		perror(SERIALDEVICE); 
		return 0; 
	}

	tcgetattr(sfd,&oldtio);
	bzero(&newtio, sizeof(newtio)); 

	newtio.c_cflag = BAUDRATE | CS8 | CLOCAL | CREAD;
	newtio.c_iflag = IGNPAR | ICRNL;
	newtio.c_oflag = 0;
	newtio.c_lflag = ICANON;

	newtio.c_cc[VINTR]    = 0;     /* Ctrl-c */ 
	newtio.c_cc[VQUIT]    = 0;     /* Ctrl-\ */
        newtio.c_cc[VERASE]   = 0;     /* del */
        newtio.c_cc[VKILL]    = 0;     /* @ */
        newtio.c_cc[VEOF]     = 4;     /* Ctrl-d */
        newtio.c_cc[VTIME]    = 0;     /* inter-character timer unused */
        newtio.c_cc[VMIN]     = 1;	/* blocking read until 1 char arrives */
        newtio.c_cc[VSWTC]    = 0;     /* '\0' */
        newtio.c_cc[VSTART]   = 0;     /* Ctrl-q */ 
        newtio.c_cc[VSTOP]    = 0;     /* Ctrl-s */
        newtio.c_cc[VSUSP]    = 0;     /* Ctrl-z */
        newtio.c_cc[VEOL]     = 0;     /* '\0' */
        newtio.c_cc[VREPRINT] = 0;     /* Ctrl-r */
        newtio.c_cc[VDISCARD] = 0;     /* Ctrl-u */
        newtio.c_cc[VWERASE]  = 0;     /* Ctrl-w */
        newtio.c_cc[VLNEXT]   = 0;     /* Ctrl-v */
        newtio.c_cc[VEOL2]    = 0;     /* '\0' */
        
        tcflush(sfd, TCIFLUSH);
        tcsetattr(sfd,TCSANOW,&newtio);

	return 1;
}

/****************************************************************************/

/* functions : 	
 *		int init_serial()			OK
 * 		double get_wavelength()			OK
 * 		int set_wavelength_grating(double l)	OK
 * 		int set_wavelength_crystal(double l)	OK
 *		int get_status()
 *		int crystal_calibrated()
 *		int wavelength_valid(double l)
 */

/* Device parameter :
 *		actual wavelength	: double wavelength
 *		start wavelength	: double start
 *		stop wavelength		: double stop
 */

//init function ; called each time before measurement
int scanmate_init(List *device_data, Scheduler *parameters) {
	
	cout << "Initializing scanmate 0.2\n";

	camac_init();	// needed to init all
	
	device_data->getvar(&start,"scanmate","start");
	device_data->getvar(&stop,"scanmate","stop");
	
	if (init_serial() == 0) {	// failed
		cout <<"Failed to initilize serial device !\n";
		return 0;
	}
	
	// can we test if REMOTE CONTROL is on ? 
	// cout << "Please set Scanmate software to REMOTE CONTROL\n";
             
	return 1;
}

// pre function : called each channel
int scanmate_pre ( List *device_data, Scheduler *parameters ) {
	int channel = parameters->act_channel();
	int chanmax = parameters->maximal_channels();
	
	wavelength = start+(stop-start)*channel/chanmax;

	//set_wavelength_grating(wavelength);
	set_wavelength_crystal(wavelength);
	device_data->putvar(wavelength,"scanmate","wavelength");
	
	return 1;
}

// main function : measurement
int scanmate ( List *device_data, Scheduler *parameters ) {
	
	// Just wait and see :-)
	return 1;
}

int scanmate_close(List *device_data, Scheduler *parameters)
{
	printf("Closing scanmate\n");
	
	// setting laser to consistent state
	 
	// restore the old port settings
	tcsetattr(sfd,TCSANOW,&oldtio);
	close(sfd);
	
	return 1;
}
