/*
 * MAX : data.h, Sam Dez 29 19:31:29 CET 2001 -sg
 *
 * This file is part of Max data acquisition software
 * Copyright (C) 1998 Christian Rosen
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* this file contains the headers for some additional utility classes
 * like list and queue classes for data storage
 * the  PacketList class is used to stack all measured data and
 * convert it to uchar * to send it via sockets
 * the "real" data storage is done by the main GUI max.cpp
 * in the class sched.cpp
 */
 
#ifndef _DATA_H_
#define _DATA_H_

#include "packet.h"
#include "definitions.h"

#include <stdio.h>
#include <string.h>
//#include <stdlib.h>
//this is one element of the list
//important : performs a deep copy of the packet before
//inserting it into the list

class PacketElement {
  friend class PacketList;
  public :
  PacketElement( dataPacket  *packet = 0, PacketElement *E = 0){
    dat_packet = new dataPacket;
    memcpy(dat_packet,packet,FULL_P_SIZE); //deep copy
    next = E;  //never forget this, or list finds no end
  }
  ~PacketElement(){
    if (dat_packet)
      delete dat_packet;
  }
  private :
  dataPacket    *dat_packet;
  PacketElement *next; //pointer to next Element in the list
};
//
//class for linked list of dataPackets
//receives measured data and sone informations as arguments
//converts it to a senPacket (see packet.h) and stores it
class PacketList {
  public :
  PacketList() { P = new dataPacket; L = NULL; }
  ~PacketList() { remove(); }
  int  empty();            //return TRUE if empty
  int  length();           //number of elements in the list
  PacketElement * last();  //the functions
  PacketElement * first(); //return pointers to move around the list
  PacketElement * next();
  // PacketElement * end();
  dataPacket    * packet();   //return pointer to the datapacket structure
  void remove();                            //kill List
  void append(int, int, int, int);          //append integer value
  void append(double, int, int, int);       //append double value
  void append(int *, int, int, int,int);    //append integer array
  void append(double *, int, int, int,int); //append integer array
  private :
  PacketElement *L;
  PacketElement *L2;  //second pointer to scroll through list
  dataPacket    *P;
};

#endif
//don't write behind the endif line
