/*
 * MAX : database.cc, Sam Dez 29 19:33:20 CET 2001
 *
 * This file is part of Max data acquisition software
 * Copyright (C) 1998 Christian Rosen
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <string.h>
#include <iostream.h>
#include <stdarg.h>

#include "definitions.h"
#include "database.h"
#include "error.hh"
#include "data.h"

//*****************************************************************************
//This is the database class
//This class is responsible for the storage of the measured data
//It is used by the main GUI, the client, only. The polld does not store data
//****************************************************************************/
Database::Database() {
  Dlist.setAutoDelete(TRUE); //when deleted really erase it
  Alist.setAutoDelete(TRUE); // dito
  period = 0;  //initialize
}
//destructor
Database::~Database() {
  remove(); //kill all databases
  delete Image;
  delete Que;
}
//
//find length of longest array
//
int Database::maxarraylength() {
  unsigned int maxlen = 0;
  for (Array_Vector_Element*  AIter = Alist.first();AIter != NULL; AIter = Alist.next()) {
    switch(AIter -> type) {
    case 1: //int
	if (AIter ->i_vec.size() > maxlen)
	  maxlen = AIter -> i_vec.size();
	break;
    case 2: //double
      if (AIter ->d_vec.size() > maxlen)
        maxlen = AIter -> d_vec.size();
      break;
    default :
      printf("%s","\033[33m");
      printf("wrong type arguments found in : Database::maxarraylength() \n");
      printf("%s","\033[0m");
      break;
      }
  }
  return maxlen;
}
//return status of database to store integer arrays
//NULL = empty
int  Database::array_empty           (void ) {
  return Alist.count();
}

int Database::database_empty         (void ) {
  return Dlist.count();
}
int Database::image_empty            (void ) {
  if (Image)
    return 1;
  return 0;
}
//
//this is the implemtation of the database class that stores
// the measured value in a list and saves them
//
void Database::add_data( int i ) {
  Scalar_Vector_Element *S= new Scalar_Vector_Element(i);
  assert(S != 0);
  Dlist.append(S);

}
//insert double variable  at beginning of the measurement data list
void Database::add_data( double i ) {
  Scalar_Vector_Element *S= new Scalar_Vector_Element(i);
  assert(S != 0);
  Dlist.append(S);

}
//insert char variable  at end of the measurement data list
void Database::add_data( char i ) {

}
//insert string variable  at end of the measurement data list
void Database::add_data( char* i ) {
  i=NULL;
}
//
//Append new array of int to database and return pointer this vector
//
vector<int> * Database::add_data( int* a,int size) {
  Array_Vector_Element *AV = new Array_Vector_Element(a,size);
  assert( AV != 0);
  Alist.append(AV);
  return &(Alist.current()-> i_vec) ; //A -> array; //return adress of data Element for plotting
}
//
//Append new array of double to database and return pointer this vector
//
vector<double> * Database::add_data( double *a, int size) {

  Array_Vector_Element *AV = new Array_Vector_Element(a,size);
  assert( AV != 0);
  Alist.append(AV);	
  return &(Alist.current() -> d_vec);  //return adress of data Element for plotting
}
//same for images
void Database::add_data(QImage *i) {
  ImageListElement *I = new ImageListElement(i,Image);
  assert( I != 0 );
  Image = I;
  printf("adding images\n");
}

//!Set the number of Data Elements per channel in database
void Database::set_N_Elements(int x) { varcounter   = x; }
//!Set the number of Arrays per channel in the database
void Database::set_N_Arrays(int x)   { arraycounter = x; }
//!set monitor mode flag TRUE = on (monitor mode) , FALSE = OFF (data acquistion mode)
void Database::set_MonMode(bool x)   { monitor_mode = x; }

int Database::length_of_list() {
  printf("length of ScData %d \n",Dlist.count());
  return Dlist.count();
}
int Database::length_of_arraylist() {
 return Alist.count();
}
int Database::length_of_imagelist() {
  ImageListElement *I = Image;
  int counter = 0;
  while ( I ) {
    I = I -> next;
    counter++;
  }
return counter;
}
//remove complete measurement data_list out of memory
void Database::remove() {
  Dlist.clear();
  Alist.clear();
  ImageListElement *I = Image;
  while ( I ) {
    ImageListElement *im_temp = I;
    I = I -> next;
    delete im_temp;
  }
  Image              = NULL; // dito
  temp_counter       = 0; //reset counters for store routine
  element_counter    = 0;
  var_index          = 0; //index for plot variables
  temp_var_index     = 0;
  arraycounter      = 0;
  array_temp_counter = 0;
  maxarraysize       = 0;
}
void Database::remove_1st() {
  Dlist.removeFirst();
}
/*! remove a given number of elements from the list,
 *!start at the beginning
 */
void Database::remove(int number) {
  if ((unsigned int) number> Dlist.count())
    number = Dlist.count();
  for (int i=0;i < number ; i++)
    Dlist.removeFirst();
} 	
void Database::array_remove_1st( void ) {
  Alist.removeFirst();
}
void Database::array_remove(int number) {
 if ((unsigned int) number > Alist.count())
   number = Alist.count();
  for (int i=0;i < number ; i++)
    Alist.removeFirst();
}
void Database::printlist() {
  printf("measured data :\n");
  printf("  \n\nNow printing  new list \n");
  Scalar_Vector_Element *Iter;
  for (Iter=Dlist.first(); Iter != 0 ; Iter = Dlist.next() ) {
    switch (Iter->type) {
    case 1: //int
      cout << "test" << endl;
      break;
    case 2: //double
      cout << "double test" << endl;
      break;
    }
  }
  unsigned int i;
  printf("  \n\nNow printing Array List  \n");
  Array_Vector_Element *AIter;
  for (AIter=Alist.first(); AIter != 0 ; AIter = Alist.next() ) {
    switch(AIter -> type) {
    case 1: //int
	for (i=0;i > AIter -> i_vec.size();i++)
	  cout << "array" << endl;
	break;
    case 2:  //double
      	for (i =0;i > AIter-> d_vec.size();i++)
	  cout << "d_array" << endl;
	break;
    }
  }
}

ImageListElement* Database::end_of_image_list(void) {
  if (Image == 0) return 0;
  ImageListElement *prev,*act;
  prev=act=Image;
  while (act) {
    prev=act;
    act = act -> next;
  }
return prev;
}
//
//list is on wrong order (due to a faster code during measurement)
//this function turns it around
//only called before data is saved
//If the measurement was aborted abnormally (cancel due to error)
//the data elements of the incompleted channel have to be removed !
//
void Database::turn_around(void ) {

}
void Database::arrayList_turn_around()
{

}
void Database::imageList_turn_around() {
  if (Image == 0 || Image -> next == 0) return;
  ImageListElement *I = Image;
  ImageListElement *Previous = 0;
  Image = end_of_image_list();
  while (I != Image ) {
    ImageListElement *Itemp = I -> next;
    I -> next = Previous;
    Previous = I;
    I = Itemp;
  }
  Image -> next = Previous;
}

void Database::save_data(const char *filename,VarType *Ptr,int counter) {
  FILE *fp;
  int varcount;
  int buff;

  printf("Length of Scalar Data List is : %d scalars.\n",Dlist.count());
  if (! Dlist.isEmpty()) {
    printf("saving scalars in : %s\n",filename);
    if ((fp = fopen(filename,"w+")) == NULL) {
      int result=Ierror(E_OPEN,"routine to save data ","");
      result = 0;
      return;
    }
    varcount = 0;
    Scalar_Vector_Element *SIterator=Dlist.first(); //Iterator points to the beginning
    while (SIterator)  //scroll through list until SIterator = NULL (points at the end)
      {
      while ( varcount != counter) {
	buff = Ptr[varcount].type;
	switch (buff) {
	case 1 : //int
	  fprintf(fp,"%i ",SIterator->i_var);
	break;
	case 2 ://double
	  fprintf(fp,"%G ",SIterator->d_var);
	  break;
	case 3 ://char
	  //	  fprintf(fp,"%c",D->c_value);
	  break;
	case 4 ://string
	  //fprintf(fp,"%s ",D->string);
	  break;
	}
	SIterator = Dlist.next();  //set Iterator to next element
	varcount++;
      }
      fprintf(fp,"\n");
      varcount = 0;
    }
    fclose(fp);
  }
}
//
//routine to save strings of integers
//each column is one string of size rows
//origin compatible format
//first column is channel number
//
void Database::save_arrays(const char *fname,ArrayType *) {
  FILE *fp;
  int arrays; //number of arrays stored
  int counter;
  unsigned int index;  //array index
  if (!Alist.isEmpty()) { //only save, if not empty
    printf("Length of Vector Database is  : %d vectors\n",Alist.count());
    printf("saving vectors in : %s\n",fname);
    if ((fp = fopen(fname,"w")) == NULL) {
      Ierror(E_OPEN,"array save routine ","");
      return;
    }
    Array_Vector_Element *AIterator = Alist.first(); //Iterator for Array List
    counter = 0;
    arrays  = 0;
    index   = 0;
    maxarraysize = maxarraylength();  //find length of longest array
    while (index < (uint) maxarraysize) {
      AIterator = Alist.first();     //reset to beginning for second vector Element
      fprintf(fp,"%d",index);
      while(AIterator) {        //loop over n-th element of all vectors
	switch (AIterator-> type) {
	case 1: //int
	  if (AIterator -> i_vec.size() > index)  //array long enough, save value
	    fprintf(fp," %d",AIterator-> i_vec[index]);
	  else
	    fprintf(fp," %d",0);
	  break;
	case 2:
	  if (AIterator -> d_vec.size() > index)  //array long enough, save value
	    fprintf(fp," %G",AIterator->d_vec[index]);
	  else
	    fprintf(fp," %d",0);
	  break;
	default  :
	  printf("%s","\033[33m");
	  printf("database.cc : error while saving arrays, wrong type !! \n");
	  printf("%s","\033[0m");
	  break;
	}
	AIterator = Alist.next();
      }	
      fprintf(fp,"\n"); //new line
      index++;	
    }	
   fclose(fp);
   }
}

void Database::save_images(const char *fname,int index,char *format) {
  QImage buffer;
  char *buf;
  buf = new char[100];
  imageList_turn_around();
  ImageListElement *I=Image;
  if (Image) {
    while (I) {
      sprintf(buf,"%s%d.%s",fname,index++,format);
      buffer = I -> Image();
      printf("saving image .....%s\n",buf);
      buffer.save(buf,format);
      I = I -> next;
     }
    //printf("finished\n\n");
  }
  imageList_turn_around();
}
