/*
 * MAX : list_polld.cc, Sam Dez 29 19:51:21 CET 2001 -sg
 *
 * This file is part of Max data acquisition software
 * Copyright (C) 1997,98 Christian Rosen
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

//list class stores the device parameters which can be seen and edited in
//the listbox
//it also provides all functions to access it -> see header file

#include <iostream.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

#include "list.hpp"
#include "error.hpp"
#include "packet.hpp"
void List::initialize() {

}
//insert int var at beginning of list
//insert does not set yscale and offset -> insert that when you need it!
void List::insert(int i_var,char *dev_name,char *var_name,int save_flag, int type) {
  Element *ZT = new Element(i_var,dev_name,var_name, save_flag, type, 0,0,L);
  assert(ZT != 0);
  L = ZT;
}
//insert does not set yscale and offset -> insert that when you need it!
//insert double var at beginning of list
void List::insert(double d_var,char *dev_name,char *var_name,int save_flag,int type) {
  Element *ZT = new Element(d_var,dev_name,var_name, save_flag, type, 0,0,L);
  assert(ZT != 0);
  L = ZT;
}

// function returns pointer to last element of the list

Element *List::End_Of_List() {
  if ( L == 0 )
    return 0;

  Element *Prev,*Act;
  Prev = Act = L;
  while (Act) {
    Prev = Act;
    Act = Act -> next;
  }
  return Prev;
}
//this function checks wether device_name is already in the list
//and returns a pointer to it.
//it returns NULL if not
Element *List::exist(char *dev_name, char *var_name) {
  Element *ListPtr; //Pointer to list element
  ListPtr = L;      //set ptr to start of list
  printf("exist: checking for %s %s \n\n",dev_name,var_name);
  while (ListPtr) {
    if ( !strcmp(dev_name,ListPtr -> device_name.c_str())  &&
		 !strcmp(var_name,ListPtr -> var_name.c_str()))
	        
      printf("exist: %10s , %10s \n",ListPtr -> device_name,ListPtr -> var_name);
      return ListPtr;
    ListPtr = ListPtr -> next;
    }
  return ListPtr;  //return NULL if not found
}
//
//add the actual priorites to the plot properties list
//this funtion is called right before the measurement starts
//needed to determine the plotwindow for the measured data
//
void List::SortPlotProperties(void) {

}
void List::append(int i_var,char *dev_name,char *var_name,int save_flag,
		  int var_type,int scale,int offset)
{
  Element *ListPtr;
  /*  Element *ZT = new Element(i_var,dev_name,var_name,save_flag, var_type,
			    scale,offset);
  //only one argument L is optional
  assert(ZT != 0);
  if ( L == 0 ) { //list is empty
    L = ZT;
    return;
    }*/
  //first check, if Element is not yet present
  if ( (ListPtr = exist(dev_name,var_name) ) != NULL ) {
    //printf("List::append[int] element x%20sx x%20sx is already in the List\n",dev_name,var_name);
    ListPtr -> enabled = save_flag; //update enabled status
       ListPtr -> i_var = i_var; //replace existing value
       //    delete ZT;
    return;
  }
  Element *ZT = new Element(i_var,dev_name,var_name,save_flag, var_type,
			    scale,offset);
  //only one argument L is optional
  assert(ZT != 0);
  if ( L == 0 ) { //list is empty
    L = ZT;
    return;
  }
  Element *End = End_Of_List();
  End->next = ZT; //append new element to list
}
void List::append(double d_var,char *dev_name,char *var_name,int save_flag,
		  int var_type,int scale,int offset )
{
  Element *ListPtr;
 
  if ( (ListPtr = exist(dev_name,var_name) ) != NULL ) {
    //printf("List::append[double] element x%20sx x%20sx is already in the List \n",dev_name,var_name);
    ListPtr -> enabled = save_flag; //update enabled status
    ListPtr -> d_var = d_var; //replace existing value
    return;
  }
  
  Element *ZT = new Element(d_var,dev_name,var_name,save_flag, var_type,
			    scale,offset ); //only one arg. L is optional
  assert(ZT != 0);
  if ( L == 0 ) { //list is empty
    L = ZT;
    return;
  } 
  Element *End = End_Of_List();
  End->next = ZT;
}
//function returns string that contains <dev_name var_name parameter>
//used for display in listbox
//first element defined by 1 and not 0
char * List::getstring(int position)
{
  int k;
  Element *ZT;
  ZT = L;	
  //  QString nameptr(ZT->device_name.size()+ZT->var_name.size()+60);
  //nameptr = "";
  char *nameptr;
  k  = 1;               //start to count from 1 and not from zero
  while (ZT && k++ < position) {
    ZT = ZT -> next;     //scroll through list to position
  }
  if (!ZT)
    return NULL;
  nameptr = new char[ZT->device_name.size()+ZT->var_name.size()+60];
  // strlen(ZT->device_name)+strlen(ZT->var_name)+60; //reserve space for namebuffer
  assert(nameptr != 0);   //error check
  switch (ZT -> type) {
  case 0:
    sprintf(nameptr,"%-20s%-20s",(const char *) ZT->device_name.c_str(),ZT->var_name.c_str());
    break;
  case 1: //int variable
    sprintf(nameptr,"%-20s%-20s%d",(const char *) ZT->device_name.c_str(),ZT->var_name.c_str(),ZT->i_var);
    break;
  case 2:
    sprintf(nameptr,"%-20s%-20s%G",(const char *) ZT->device_name.c_str(),ZT->var_name.c_str(),ZT->d_var);
    break;
  case 3: //char
    /*    nameptr.sprintf("%-20s%-20s%",(const char *) ZT->device_name.c_str(),ZT->var_name.c_str());
    string dummy;
    dummy.sprintf("%-10s",ZT->s_var);
    nameptr+=dummy;
    */
    break;
  case 4: //string
    /*    nameptr.sprintf("%-20s%-20s%",(const char *) ZT->device_name.c_str(),ZT->var_name.c_str());
    string dummy;
    dummy.sprintf("%-10s",ZT->s_var);
    nameptr+=dummy; 
    */   
    break;
  }
  return nameptr;
}
//
//following functions set the variable value of an element in the list defined by it's
//position
//used for device data listbox editor
//
void List::putvar(int value,int position) {
 Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
  while (ZT && pos++ < position)
     ZT=ZT->next;
  ZT->i_var = value;

}
void List::putvar(double value,int position) {
 Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
  while (ZT && pos++ < position)
     ZT=ZT->next;
  ZT->d_var = value;
}
void List::putvar(char *string,int position) {
 Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
  while (ZT && pos++ < position)
     ZT=ZT->next;
  strcpy(ZT->s_var,string);
}
//
//the following functions are used for the listbox feature
//they give info about elements defined by their positions in the list
//
int List::get_type(int position)
{
  Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
  while (ZT && pos++ < position)
     ZT=ZT->next;
  if (!ZT)
    return -1;
  return ZT->type;
}
int List::get_store_flag(int position)
{
  Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
  while (ZT && pos++ < position)
     ZT=ZT->next;
  if (!ZT)
    return -1;
  return ZT->storeflag;
}
int List::get_plot_flag(int position)
{
  Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
  while (ZT && pos++ < position)
     ZT=ZT->next;
  if (!ZT)
    return -1; //end of list reached, value not found, index too high
  return ZT->enabled;//if zero, variable is not to be plotted
}
//identical to get_plot_flag
int List::get_enabled_flag(int position)
{
  Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
  while (ZT && pos++ < position)
     ZT=ZT->next;
  if (!ZT)
    return -1; //end of list reached, value not found, index too high
  return ZT->enabled;//if zero, variable is not to be plotted
}
void List::set_plot_flag(int position,bool state) {
  Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
  while (ZT && pos++ < position)
     ZT=ZT->next;
  if (ZT)
    ZT->plotflag=state;
}
void List::set_enabled_flag(int position,bool state) {
  Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
  while (ZT && pos++ < position)
     ZT=ZT->next;
  if (ZT)
    ZT ->enabled=state;
}
//return device name as function of position in List
char *List::get_name(int position)
{
Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
 while (ZT && pos++ < position)
     ZT=ZT->next;
 if (!ZT)
   return NULL;
 return (char *) ZT->device_name.c_str();//if zero, variable is not to be plotted
}
//return the variable name of device at position i
//used for the Init Plotwidgetrotuine
char *List::get_var_name(int position)
{
Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
 while (ZT && pos++ < position)
     ZT=ZT->next;
 if (!ZT)
   return NULL;
return ZT -> var_name.c_str();
}
//returns number of variables stored into memory for device
//specified by name
//uses the "data" field in the initfile for this
//plot window needs that number to set up windows
int List::get_number_plotvars(char *dev_name )
{
  int counter;
  Element *ZT;
  ZT = L;        //ZT points to beginning of list
  counter = 0;
  while (ZT) {
    if (!strcmp (ZT->device_name.c_str(),dev_name) && ZT -> enabled)
      counter ++; //save data flag for device "namebuffer" is nonzero
    ZT = ZT -> next; //scan through list until name is found
  }
return counter;
}
//Returns string with all names of plotvariables of device *dev_name
//needed for plotroutine to name the plots
string List::get_plotvar_namelist(char *dev_name)
{
  string namelist;
  Element   *ZT;
  ZT = L;
  while (ZT) {
    if (!strcmp (ZT->device_name.c_str(),dev_name) && ZT -> enabled) {
       namelist += ZT -> var_name;
       namelist += " ";
    }
     ZT = ZT -> next;
  }
  return namelist;
}
int List::getint(int position)
{
  Element *ZT;
  ZT = L;
  int pos;
  pos = 0;
  while (pos++ < position)
    ZT=ZT->next;
  return ZT->i_var;
}

double List::getdouble(int position)
{
 Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
  while (pos++ < position)
    ZT=ZT->next;
  return ZT->d_var;
}

char  List::getchr(int position)
{
 Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
  while (pos++ < position)
    ZT=ZT->next;
  return ZT->c_var;
}

char * List::getstr(int position)
{
 Element *ZT;
  int pos;
  ZT = L;
  pos = 0;
  while (pos++ < position)
    ZT=ZT->next;
  return ZT->s_var;
}
//
//function returns the int value of the type flag of device
//
char* List::get_data_type(char *dev_name) {

  char *typelist;
  typelist = new char[1000];
  assert (typelist != 0);
  Buffer_pointer = L;
  int pos;
  //scroll through list until first element of device_name is found
  for (pos = 0; pos < 1000; pos ++)
    typelist[pos]='\0';
  pos = 0;
  while (Buffer_pointer) {
    if (strcmp(dev_name,Buffer_pointer->device_name.c_str())==0) {
      switch  (Buffer_pointer -> enabled) {
      case 1 : //int
	typelist[pos++]='i';
	break;
      case 2 :
	typelist[pos++]='d';
	break;
      case 3 :
	typelist[pos++]='c';
	break;
      case 4 :
	typelist[pos++]='s';
	break;
      }
    }
  Buffer_pointer = Buffer_pointer -> next; //go to next element
  }
  typelist[pos]='\0';
return typelist;
}
void List::reset_data_buffer() {
  BufferPointer = L;
}
void List::ResetHelpPointer() {
  BufferPointer = L;
}
//set secondary pointer to next element in the lsit
//return zero when @ end of list
int List::AdvanceHelpPointer() {
  /*  if (BufferPointer )
    BufferPointer = BufferPointer -> next;
  else
    return FALSE;
    return TRUE;*/
  if (BufferPointer -> next) {
    BufferPointer = BufferPointer -> next;
    return TRUE;
  }
  else
    return FALSE;

}
//return sevice data of element <device_name>
//in packet structure for socket transfer
//used for sending devData to polld
devDataPacket *List::GetPacketElement(char *dev_name) {
  Element *ListPtr;
  ListPtr = L;     //set pointer to
  devDataPacket *Packet = new devDataPacket;
  assert (Packet !=0 );
  char *buffer;
  buffer = new char[MAX_NAME_LENGTH+MAX_VAR_LENGTH];        //allocate mem for strings and clear it
  while (ListPtr) {
    if (strcmp(dev_name,BufferPointer -> device_name.c_str()) == 0) {
      strncpy((char *) Packet ->device_name,
	      BufferPointer -> device_name.c_str(),MAX_NAME_LENGTH);
      strncpy((char *) Packet -> var_name  ,
	      BufferPointer -> var_name.c_str()   ,MAX_NAME_LENGTH);
      Packet -> enabled = (unsigned char) BufferPointer -> enabled;
      switch(BufferPointer -> type){ //check for variable type
      case 0:
	//	printf("null found in %s \n",ListPtr->var_name);
	break;
      case 1://int
	sprintf( buffer," %d",BufferPointer -> i_var);
	strncpy(( char *) Packet -> value, buffer,MAX_VAR_LENGTH);
	Packet -> type = (char ) BufferPointer -> type;
	break;
      case 2: //double
	sprintf( buffer,"%G",BufferPointer -> d_var);
	strncpy(( char *) Packet -> value, buffer,MAX_VAR_LENGTH);
	Packet -> type = (char ) BufferPointer -> type;
	break;
      case 3: //char;
	printf("ERROR : List::GetPacketElement : no char variable !!\n");
	break;
      case 4: //string
	printf("ERROR : List::GetPacketElement : no string variable !!\n");
	break;
      default :
	break;
      }
    }
    ListPtr = ListPtr -> next;
  }
  delete buffer;
  return Packet;
}
//return packet structure with elements
//pointed to by the secondary help pointer
//used for sending devData to polld
devDataPacket *List::GetNextPacket(void ) {
  devDataPacket *Packet = new devDataPacket;
  assert (Packet !=0 );
  char *buffer;
  buffer = new char[MAX_NAME_LENGTH+MAX_VAR_LENGTH];        //allocate mem for strings and clear it
  if (BufferPointer) {
    strncpy((char *) Packet ->device_name,
	    BufferPointer -> device_name.c_str(),MAX_NAME_LENGTH);
    strncpy((char *) Packet -> var_name,
	    BufferPointer -> var_name.c_str(),MAX_NAME_LENGTH);
    Packet -> enabled = (char ) BufferPointer -> enabled; //copy enabled flag
    switch(BufferPointer -> type){ //check for variable type
    case 0:
      //	printf("null found in %s \n",ListPtr->var_name);
      break;
    case 1://int
      sprintf( buffer," %d",BufferPointer -> i_var);
      strncpy(( char *) Packet -> value, buffer,MAX_VAR_LENGTH);
      Packet -> type = '1';
      break;
    case 2: //double
      sprintf( buffer,"%G",BufferPointer -> d_var);
      strncpy(( char *) Packet -> value, buffer,MAX_VAR_LENGTH);
      Packet -> type = '2';//(char ) BufferPointer -> type;
      break;
    case 3: //char;
      printf("ERROR : List::GetPacketElement : no char variable !!\n");
      break;
    case 4: //string
      printf("ERROR : List::GetPacketElement : no string variable !!\n");
      break;
    default :
      break;
    }
  }
  delete buffer;	
  return Packet;
}

//
//return string of all parameters of device_name separated by string
//Do not return the active variable (because only one parameter in the line
//this is checked by the int List::is_active(Name) function
//
char * List::getAllDeviceParameters(char *dev_name )
{
  // char* ParameterStr;
  Element *ListPtr;
  char *buffer;
  char *buf;
  buf = new char[100];
  ListPtr = L;                  //set pointer ZT to start of List
  buffer = new char[10000];     //allocate memfor strings and clear it
  assert(buffer !=0);           //error check for misallocation
  memset(ParameterStr,0,strlen(ParameterStr));
  strcpy(ParameterStr,"Device :  ");
  strcat(ParameterStr,dev_name); //add device name in the beginning
  strcat(ParameterStr,"\n");
   while (ListPtr) { //scroll through complete list
    if (strcmp(dev_name,ListPtr -> device_name.c_str()) == 0) { //only complete string for specified device
      sprintf(buf,"%-20s",ListPtr -> var_name.c_str());
      strcat(ParameterStr,buf);//ListPtr -> var_name
      strcat(ParameterStr,"  ");
       //THIS SWITCH SHOULD BE REMOVED IN THE FUTURE
       // -> OVERLOAD OPERATORS AND USE TEMPLATE LIST CLASS
      switch(ListPtr -> type){ //check for variable type
      case 0:
	//	printf("null found in %s \n",ListPtr->var_name);
	strcat(ParameterStr,"\n"); //only add CR
	break;
       case 1://int
	 sprintf( buffer," %d",ListPtr -> i_var);
	 strcat(ParameterStr,buffer);
	 if (ListPtr -> enabled) //check if plot variable
	   strcat(ParameterStr,"  stored int");
	   strcat(ParameterStr,"\n");
	 break;
       case 2: //double
	 sprintf( buffer," %G",ListPtr -> d_var);
	 strcat(ParameterStr,buffer);
	 if (ListPtr -> enabled) //check if plot variable
	   strcat(ParameterStr,"  stored double");
	 strcat(ParameterStr,"\n");
	 break;
       case 3: //char;
	 sprintf(buffer,"%c",ListPtr -> c_var);
	 strcat(ParameterStr,buffer);
	 if (ListPtr -> enabled)  //check if plot variable
	   strcat(ParameterStr,"  saved char_var");
	 strcat(ParameterStr,"\n");
	 break;
       case 4: //string
	 sprintf(buffer,"%s",ListPtr -> s_var);
	 strcat(ParameterStr,buffer);
	 if (ListPtr -> enabled) //check if plot variable
	   strcat(ParameterStr,"  saved string-var");
	 strcat(ParameterStr,"\n");
	 break;
       default :
	 strcat(ParameterStr, "illegal type specifier found !");
	 break;
       }
    }
    ListPtr = ListPtr->next; //set pointer to next element
    }
  strcat(ParameterStr,"\n");
  return  ParameterStr;
}
int List::getvar(int *i_var,char *dev_name,char *var_name)
{
  *i_var =0;
  Element *ZT = L ;
  while ( ZT && varcomp(ZT, dev_name,var_name))
    ZT=ZT->next;
  if (!ZT) {
    printf("element %s not found\n",var_name);
    return 0;
  }
  *i_var=ZT->i_var;
  return 1;
}

int List::getvar(double *d_var,char *dev_name,char *var_name)
{
  *d_var =0;
  Element *ZT = L ;
  while ( ZT && varcomp(ZT, dev_name,var_name))
    ZT=ZT->next;
  if (!ZT) {
    printf("element %s not found\n",var_name);
    return 0;
  }
  *d_var=ZT->d_var;
  return 1;
}

int List::putvar(int i_var,char *dev_name,char *var_name)
{
  Element *ZT = L ;
  while ( ZT && varcomp(ZT, dev_name,var_name))
    ZT=ZT->next;
  if (!ZT) {
    printf("element %s not found!\n",var_name);
    return 0;
  }
  ZT->i_var = i_var;
  return 1;
}

int List::putvar(double d_var,char *dev_name,char *var_name)
{
 Element *ZT = L ;
  while ( ZT && varcomp(ZT, dev_name,var_name))
    ZT=ZT->next;
  if (!ZT) {
    printf("element %s not found!!!\n",var_name);
    return 0;
  }
  ZT->d_var = d_var;
  return 1;
}
//
//test the status of a "channel' of the device
//return true if the enabled flag is set
//e.g. if the user has switched on this "channel"
//
int List::enabled(char *dev_name,char *var_name) {
  Element *ZT = L ;
  while ( ZT && varcomp(ZT, dev_name,var_name))
    ZT=ZT->next;
  if (!ZT) {
    printf("element %s not found!!!\n",var_name);
    return 0;
  }
  return ZT -> enabled;
}
//return 0 : no store-variable
//return 1 : store variable, not enabled
//       2 : store var enabled
//return 3 : store var enabled and plot
//
int List::plotstatus(int position) {
  int     k  = 1;               //start to count from 1 and not from zero
  Element *ZT = L;
  while (ZT && k++ < position) {
    ZT = ZT -> next;     //scroll through list to position
  }
  if (!ZT)
    return -1;
  if (ZT -> enabled == -1)
    return 0;
  if (ZT -> plotflag && ZT -> enabled )
    return 3;
  return -1;
}
void SortPlotProperties(void) {

}

//
//removes complete list out of memory
//
void List::remove() {
  Element *ZT = L;
  while (ZT ) {
    Element *tmp = ZT;
    ZT= ZT->next;
    delete tmp;
  }
  L  = 0;
}
//
//this function tells you wether the specified device is an active device
//
int List::is_active(char *dev_name) {
  Element *ZT = L;
  while ( ZT && varcomp(ZT, dev_name,"active"))
    ZT = ZT -> next;
  if (!ZT)
    return -1;
  return 1;
}

int List::remove(int i_var) {
  Element *ZT = L;
  int Number = 0;
  while ( ZT && ZT->i_var == i_var) {
    Element *tmp=ZT->next;
    delete ZT;
    ++ Number;
    ZT = tmp;
  }
  if ((L=ZT) == 0 ) {
    return Number;
  }

  Element *Prev = ZT;
  ZT=ZT->next;
  while ( ZT ) {
    if ( ZT ->i_var == i_var ) {
      Prev->next = ZT->next;
      delete ZT;
      ++ Number;
      ZT = Prev->next;
    }
    else {
      Prev = ZT;
      ZT = ZT ->next;
    }
  }
  return Number;
}

int List::remove(double d_var) {
  Element *ZT = L;
  int Number = 0;
  while ( ZT && ZT->d_var == d_var) {
    Element *tmp=ZT->next;
    delete ZT;
    ++ Number;
    ZT = tmp;
  }
  if ((L=ZT) == 0 ) {
    return Number;
  }
  Element *Prev = ZT;
  ZT=ZT->next;
  while ( ZT ) {
    if ( ZT ->d_var == d_var ) {
      Prev->next = ZT->next;
      delete ZT;
      ++ Number;
      ZT = Prev->next;
    }
    else {
      Prev = ZT;
      ZT = ZT ->next;
    }
  }
  PlotPropList -> clear();
  return Number;

}
void List::printlist()
{
  printf( " ( ");
  for (Element *ZT = L ; ZT ; ZT = ZT -> next)
   ; //cout << ZT->d_var << ZT->device_name << ZT->var_name << endl;
  //    printf("%f %s  %s, ", ZT->d_var,ZT->device_name,ZT->var_name );
  printf(" )\n");
}
int List::initlist(char *filename)
{
  const char  *parse_data[5]={"int","double","device","active","store"};

  printf("initlist called\n");
  PlotProps *P;
  FILE *fp;
  int no_devices;
  int result = 1;
  int enabled = 0;
  char var_name[20];
  char type[10];
  char device_name[20];
  char parameter[10];
  char data[50];
  char comment[300];
  //first kill list, if there is one

  index = 0; //plotvariable index
  if ((fp=fopen(filename,"r"))==NULL){
    printf("Could not open file\n");
    int result = Ierror(E_FILE_NOT_FOUND,filename,"");
    result = 1;
    return result; //exit(0);
  }
  remove();
  PlotPropList -> clear(); //erase list
  // PlotPropList   = new QList<PlotProps>;
  //assert(PlotPropList != 0);
  printf("parsing parameter initfile....\n");
  no_devices = 0;
  while (fscanf(fp,"%s",data)!=EOF) {
    i=0;
    if (strncmp(data,"#",1)==0) {         //check for comment line
           fgets(comment,300,fp);       //read until end of line
	   //	   printf("comment %s found \n",comment);
    }
    else {
      while (strcmp(parse_data[i++],data) && i<6) //scan through type list and do error check
	;
      switch(i) {
      case 1://int
	if(no_devices) {
	  fscanf(fp,"%s",var_name);
	  fscanf(fp,"%s",parameter);
	  append((int) atoi(parameter),device_name,var_name,0,1,0,-1  );
	}
	break;
      case 2://double
	if (no_devices){
	  fscanf(fp,"%s",var_name);
	  fscanf(fp,"%s",parameter);
	  append((double) atof(parameter),device_name,var_name,0,2,0,-1);
	}
	break;
      case 3://device
	fscanf(fp,"%s",device_name);
	printf("new device %10s\n",device_name);
      no_devices++;
      break;
      case 4: //active
	append(0,device_name,"active",0,0,0,-1);
	//Add Node with var_name active and var_type active and value 0
	break;
	//
	/*new format :
	 * var <name>
	 *options.....
	 */
      case 5: //add plot element for storage of plot parameters
	//printf("plot element found \n");
	fscanf(fp,"%s",var_name);  //scan name of plotvar
       	P = new PlotProps(device_name,var_name,index);
	if ((enabled = initPlotOptions(fp,P))==-1) {//enabled ==0 -> channel disabled
	  printf("Leaving programm at devices %s,%s \n",device_name,var_name);
	  return -1;
	  }
	PlotPropList -> append(P);
	//	printf("length of plotprolist %d\n",PlotPropList -> count());
	append((int) atoi(type),device_name,var_name,enabled,1,1,1);//data type = 1 for int
	index++;
	break;
      case 6:
	if (result ) {
	  printf("\n\n\n!!!!!!error found in the initfile!!!!!!!!!!! \n");
	  printf("wrong variable type after %s %s \n",device_name,var_name);
	  printf("I will quit this application !!! \n");
	}
	result = 0;
	return -1;
	break;
      default :
	break;
      }
    }
  }
  fclose(fp);
  //  printlist();
return result;
}
//Var field in initfiled detected, now read all plot options
int List::initPlotOptions(FILE *fp,PlotProps *PP)
{
  int    enabled=0;
  int    p=0;
  bool   flag=FALSE;
  char   data[500];
  char   comment[500];
  char   idat[sizeof(int)];
  //These options are recognized in the initifile -> definitions.hpp
  const char *plot_options[20]={"P_MIN_SIZE_X","P_MIN_SIZE_Y","Y_TICKS",
				"X_TICKS","TICKSIZE","TICKPOS","TICKPOS_X",
				"HGRIDLINES","VGRIDLINES","Y_UPPER_LIMIT",
				"Y_LOWER_LIMIT","AUTOSCALE","AS_INTERVAL",
				"SOLIDLINE","POINTS","ENABLED","PLOT",
				"P_LEFT_BORDER","P_UPPER_BORDER","\0"};
  printf("plot found\n");
  while (strncmp(data,"{",1) != 0)
    fscanf(fp,"%s",data); //read until "{" is found
  fscanf(fp,"%s",data); //read first line after "{"
  if (strncmp(data,"#",1) == 0) { //comment detected
    fgets(comment,300,fp);        //read til end of line
    fscanf(fp,"%s",data);         //read next line
  }
  while (strncmp(data,"}",1) != 0) { //read until end of options field reached
    p=0;

    while(strcmp(plot_options[p++],data) && p < 18) //scan through options list
      ;
    switch(p) {
    case 1 : //x_size of widget
      fscanf(fp,"%s",idat);
      PP -> Ps.pminx = atoi(idat);
      break;
    case 2 :  //y_size
      fscanf(fp,"%s",idat);
      PP -> Ps.pminy = atoi(idat);
      break;
    case 3 :
      fscanf(fp,"%s",idat);
      PP -> Ps.yticks = atoi(idat);
      break;
    case 4 :
      fscanf(fp,"%s",idat);
      PP -> Ps.xticks = atoi(idat);
     break;
    case 5 :
      fscanf(fp,"%s",idat);
      PP -> Ps.ticksize = atoi(idat);
      break;
    case 6 :
      fscanf(fp,"%s",idat);
      PP -> Ps.tickpos = atoi(idat);
      break;
    case 7 :
      fscanf(fp,"%s",idat);
      PP -> Ps. tickposx = atoi(idat);
      break;
    case 8 : //horizontal gridlines
      fscanf(fp,"%s",idat);
      if (strcmp("TRUE",idat)==0)
	PP -> Ps.hgridlines = TRUE;
      else
	PP -> Ps.hgridlines = FALSE;
      break;
    case 9 :  //vertical gridlines
      fscanf(fp,"%s",idat);
      if (strcmp("TRUE",idat)==0)
	PP -> Ps.vgridlines = TRUE;
      else
	PP -> Ps.vgridlines = FALSE;
      break;
    case 10 :  //Y_UPPER_LIMIT
      fscanf(fp,"%s",idat);
      PP-> Ps.yupper =  atof(idat);
      break;
    case 11 : //Y_LOWER_LIMIT
      printf("YLOWER \n");
      fscanf(fp,"%s",idat);
      PP-> Ps.ylower = atof(idat);
      break;
   case 12 : //autoscale enabled
     fscanf(fp,"%s",idat);
      if (strcmp("TRUE",idat)==0)
	PP -> Ps.autoscale =TRUE;//PP -> autoscale  = TRUE;
      else
	PP -> Ps.autoscale =FALSE;//	PP -> autoscale  = FALSE;
      break;
   case 13 : //autoscale intervall
     fscanf(fp,"%s",idat);
      PP -> Ps.as_interval = atoi(idat);
      break;
    case 16 : //channel enabled statement
      fscanf(fp,"%s",idat);
      if (strcmp("TRUE",idat)==0)
	enabled = 1;
      else
	enabled = 0;
      break;
    case 17 : //plot statement
      fscanf(fp,"%s",idat);
      if (strcmp("TRUE",idat)==0)
	flag = TRUE;
      else
	flag = FALSE;
      break;
    case 18 : //P_LEFT_BORDER
      printf("P_LEFT");
      fscanf(fp,"%s",idat);
      PP -> Ps.pleftborder = atoi(idat);
      break;
    case 19 : //P_UPPER_BORDER
      printf("P_UPPER");
      fscanf(fp,"%s",idat);
      PP -> Ps.pupperborder = atoi(idat);
      break;
    case 20 :
      printf("****************************************\n");
      printf("***   Error in plotstatement !!!     ***\n");
      printf("** I don't understand %s !! **\n",data);
      printf("****************************************\n");
      return -1;
    break;
    default :
      printf("Error :\n I don't understand this part in the plot statement:");
      printf("%s , please correct your spelling in the devices initfile. \n",data);
      return -1;
      break;
    }
    fscanf(fp,"%s",data);
    if (strncmp(data,"#",1) == 0) { //comment detected
      fgets(comment,300,fp);        //read til end of line
      fscanf(fp,"%s",data);         //read next line
    }
  }
  //set the default values in case PLOT, ENABLED are not given
  PP -> plotflag  = flag;
  PP -> storeflag = enabled;
  return enabled; //default is disabled
}

//help functions, don't belong in this file
int List::varcomp(Element *currentPtr,char *device_name, char *var_name)
{
  x=strcmp(var_name,currentPtr->var_name.c_str());
  y=strcmp(device_name,currentPtr->device_name.c_str());
  if (!x && !y)
    return 0;
  else
    return -1;
}

















