/*
 * MAX : plot.h, Sam Dez 29 20:00:12 CET 2001 -sg
 *
 * Plot widget to plot measured data for max data acquisition software
 * header file
 *
 * This file is part of Max data acquisition software
 * Copyright (C) 1997,98 Christian Rosen
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

//#include "qtincludes.h"
#include "definitions.h"
#include "packet.h"

//qt includes
#include <qwidget.h>
#include <qpainter.h>
#include <qlist.h>
#include <qlabel.h>
#include <qmenubar.h>
#include <qcolor.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qtoolbar.h>
#include <qprinter.h>

//Standard template Lib (STL)  includes
#include <vector>

#ifndef _PLOT_H_
#define _PLOT_H_
/************************************
 * Options widget to set y - scales *
 ***********************************/
class ScaleW : public QWidget
{
  Q_OBJECT
public:
  ScaleW(char *,double, double/*,QWidget *parent = 0,const char* name = 0*/);
signals :
  void SIG_setlimits(double,double);
public slots :
  void  okpressed();
  void  cancelpressed();
private :
  QLineEdit   *limEdit;
  QLineEdit   *limEdit2;	
  QLabel      *label1;
  QLabel      *label2;	
  QLabel      *headline;
  QPushButton *ok;
  QPushButton *cancel;
};
//defines for the plotwidget
#define PWidget_X    200
#define PWidget_Y    300
//
//This structure contains one point to plot
//This class contains a vector of this structure to handle plotdata
//
typedef struct {
  int     x;
  double  y;
} point;


class Plot : public QMainWindow //public QWidget
{
  Q_OBJECT;
public :
  Plot(QWidget *parent = 0, const char *name =0,char *devname =0,char *varname =0,int chan=1000 ,int m = FALSE);
  ~Plot();
  void plotPoint(int ,int );         //plot point at channel x and y
  void plotPoint(int, double);       //plot double
  void plotPoint(int ,int *,int);    //plot array of channel x
  void plotPoint(int, double *,int); //parameters are : channel, array,length
  void plotPoint(int/* chan=0*/, vector<int>* ,int );    //store vector of int in local plotbuffer
  void plotPoint(int, vector<double> *,int); //store vector of double in local plotbuffer
  void setYAxis(int ,int ,int);      //type, lower , upper limit
  void setXAxis(int, int, int);      //type , lower , upper limit -> convert channel numbers here
  void setXText(char *);
  void setYText(char *);
  void totalScalingOn();  //graph scales with windowsize
  void totalScalingOff(); //graph gets scrollbars
  void setLinestyle(int );
  void setTitle(char *);
  void setColors(QColor, QColor,QColor );
  void SetAllOptions(PlotSettings *);
  void refresh(void);  //force update of plotwidget -> draw the data
public slots:
  void SLOT_set_scales( double, double ); //called from setscales widget
  void setscales();
  void updateOn();
  void updateOff();
  void autoscaleOn();
  void autoscaleOff();
  void toggleAS();
  void hgridOff();
  void hgridOn();
  void vgridOn();
  void vgridOff();
  //  void setLineStyle(int );

private slots:	
  void save();   //save the data currently visible
  void print();//prints the data currently visible
  void kill();

protected:
  void resizeEvent( QResizeEvent * );
  void paintEvent( QPaintEvent *);
  void mouseMoveEvent( QMouseEvent *);
  void DrawPlot( QPainter *);
  void DrawAxes( QPainter *);

private slots :
  void del();

private :
  //first, Qt stuff :
  QPrinter *printer;
  QToolBar *fileTools;

  QMenuBar *menu;
  QLabel   *Title;
  QLabel   *XLabel;
  QLabel   *YTitle;
  QLabel   *RightSpace;
  QLabel   *PlotLabel;  // this label contains the main pixmap
  QLabel   *XAxisLabel; //these labels contains the axis pixmaps
  QLabel   *YAxisLabel;
  QLabel   *xval;       //cursorposition on plotwindow
  QLabel   *yval;
  QLabel   *as;         //show autoscale status
  // color settings
  QColor  axiscolor;  //color of axis and ticks
  QColor  linecolor;  //color of gridlines
  QColor  background;
  QColor  labelcolor;
  QColor  arraylinecolor; //color if dataset is array
  QColor  datacolor; //color of datase
  ScaleW  *SW;       //Widget to enter scaling
  //
  //regular variables
  //
  bool draw_axis;  //for repaint event, don't always redraw the axes
  bool autoscale;
  bool hgrid;      //draw horizontal gridlines
  bool vgrid;      //draw vertical gridlines
  int type;           //type of data in database (int or double) for save routine int = 0, double =1
  int l_plimit;
  int as_interval;
  int as_counter;     //counter for number of points to be above upper limit
  int las_counter;    //dito below lower limit
  int pXsize,pYsize;  //sizes of plotwindow
  int pXpos,pYpos;    //upper left corent of plotarea
  int xticks;         //number of ticks for the axes
  int yticks;
  double ylow,yhigh;
  double xlow,xhigh;
  double xscale,yscale;
  double scale;
  double offset;
  int  PLeftBorder,PUpperBorder;
  int ticksize;
  int tickpos,tickposx;
  int monitor_mode; //if monmode set chan = 1000 + replot after 1000 channels
  bool upd;   //this flag indicates wether window should be cleared before next plot
  int  arrays;   		//indicator where to take data from
  vector<int>    *iVecPtr; 	//Pointer to vector of integers
  vector<double> *dVecPtr; 	//Pointer to vector of doubles
  int    array_length;
  //variables to specifiy the plot  data
  int deviceIndex;
  int varIndex;
  int linestyle;      //draw points or lines -> see defintions.h
  char *devName;
  char *varName;
  double dmax,dmin;
  int  max,min;
  int  pminsizex;
  int  pminsizey;
  int  last_x;        //intervall in x-coordinates data has to be refreshed in
  int  new_x;
  int  l_plotlim;     //index of database to plot from
  char  *va_name;     //name of variable to be plotted
  vector<point>  plotVec; //this vector contains all the plot data
};


#endif
