/*
 * MAX : polld.cc, Son Dez 30 00:11:07 CET 2001 -sg
 *
 * - polling deamon (polld) -	This is the Server
 * this class calls the device routines via the scheduler class it 
 * performs the measurement and is controlled via sockets by the client
 *
 * This file is part of Max data acquisition software
 * Copyright (C) 1998 Christian Rosen
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "polld.h"

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <signal.h>
#include <sys/types.h>
#include <fcntl.h>

//#define DEBUG_POLLD

int Server_Socket_Connect(void) ;

Polld::Polld(int argc,char *argv[]) :
  QApplication(argc,argv)
{
  //lists to stack packets before sending
  //packets are only sent within defined intervalls
  //in between data packets are buffered in linked lists
  DevSQueue  = new  QList<devDataPacket>; //send buffer for device parameters
  DevRQueue  = new  QList<devDataPacket>; //receive buffer
  DevRQueue -> setAutoDelete(TRUE);
  DevSQueue -> setAutoDelete(TRUE);
  d_packet   = new devDataPacket;   //allocate mem for d_packet
  Schedule   = new Scheduler;       //Scheduler class for device database
  Devices    = new List;
  QApplication::connect(Schedule,SIGNAL(Sig_Error(char *)),
			this,SLOT(SendError(char *)));
  initialize(); //reset all values
   
	//first, do network stuff
  	
	//if no argument given connect on localhost
#ifdef DEBUG_POLLD
  	cout<<"Polld : Ready and listening for Client socket requests ...\n";
#endif	

	char *host="localhost";
	if (argc == 2) {
		host = argv[1];
	}

	if (socketConnect(host) == -1) {
    		printf("Polld : error while establishing network connection\n");
    		printf("bailing out \n");
    		::exit(-1);
  	}
  
	DataReceivedNotifier = new QSocketNotifier(data_sfd,
				QSocketNotifier::Read,this);
 	QObject::connect(DataReceivedNotifier,SIGNAL(activated(int)),
				this,SLOT(dataReceived()));

  //connect QSocket Notifier to handle commands arriving from the client (max)
  CommandNotifier = new QSocketNotifier(cmdr_sfd,QSocketNotifier::Read,this);
  QObject::connect(CommandNotifier,SIGNAL(activated(int)),
  		   this,SLOT(commandReceived()));

  connect(this,SIGNAL(start()),
	  this,SLOT(startMeasurement()));

  /*
   *set up timer for internal trigger and connect timer even
   */
  intTrigger = new QTimer(this);   //timer for update of channel number widget
  send1 = new QTimer(this);   //timer for internal trigger

  connect(intTrigger,SIGNAL(timeout()),
	  this,SLOT(TriggerTimerEvent()));
  connect(send1,SIGNAL(timeout()),
	  this, SLOT(sendParameters()) );
}
Polld::~Polld()
{
  delete &Devices;
  delete &Priority;
  delete Schedule;
  delete DevSQueue;
  delete DevRQueue;
}

void Polld::initialize(void ) {
#ifdef DEBUG_POLLD
	cout<<"Polld : initializing polling deamon and it's databases......\n";
#endif
  	//  busy = FALSE;
	suspend          = FALSE;
	readytosend      = FALSE;
	m_running        = FALSE;
	continue_flag    = FALSE;
	send_period      = 0;
	wait             = 1;
	max_channels     = 0;
	max_integrations = 0;
	triggermode      = 1;
	trigger_period   = INT_TRIGGER_PERIOD;
	stopflag         = 0;
	pause_flag       = 0;
	data_send_interval = DATA_SEND_INTERVALL; //timer period in ms
	Devices->remove(); //Clear device parameter database
	Priority.Pr_remove(); //remove priorities
	Schedule -> set_triggermode(1); //set external trigger
}
/****************************************************/
/*this routine performs the measurement             */
/*it is called after START-command has been received*/
/****************************************************/
void Polld::startMeasurement() {
	//Devices->printlist();
	int init_status;
	send1 -> start(data_send_interval,TRUE);
	if (!continue_flag) {
		cout<<"\n\n*********************************************\n";
    		cout<<"*         Starting measurement ....         *\n";
		printf("* integrations : %-5d, Channels : %-5d    *\n",
	   		Schedule-> maximal_integrations(),max_channels);
		cout<<"*********************************************\n\n";
    		Schedule ->remove(); //clear database for new measurement
  	}
  	else {
		cout<<"\n\nContinue measurement .....\n";
		printf("Integrations : %d ,Channels : %d",
				Schedule -> maximal_integrations(),
	   			Schedule -> act_channel());
    		Schedule -> remove_continue(); //clear database for continue
  	}
  
	int status=1;   //return status of devices
  	init_status = 1;
  	
	if (Schedule -> trigger_mode()) { //TRUE -> external trigger
    		if (!m_running) {// measurement running already ?
      			m_running = TRUE;
      			// initialize all devices first, 
			// if status = FALSE -> no measurement

      			if (!continue_flag) {
				init_status= Schedule -> init_measurement(Devices,Priority) ;
				printf("Polld(Measurement) : devices initialized\n");
      			}	

      			if (!Schedule->monitor_mode()) //if not in monitor mode
				continue_flag = 1; //enable continue
      			else
				continue_flag = 0;

      			m_running = 1;       //disable other start events
      			stopflag  = 1;       //enable stop button
      			save_flag = 0;       //fresh data will be present

      			Schedule ->reset_queue();
      			//start the measurement
      			if (init_status) {  //only perform measurement if initialization was o.k.
				if (!Schedule->monitor_mode() ) { //data acquisition mode
					while (Schedule->act_channel() <
		 				Schedule->maximal_channels() && stopflag && status) {
						if ( !pause_flag ) {
	      						Schedule->inc_channel(); //increment  channel number
							Schedule->reset_integrations();
							do {
								status = Schedule->measure(Devices,Priority);
								Schedule->inc_integrations();
							} while (Schedule->act_integrations() <
								Schedule->maximal_integrations() && status );
	    					}
	  	  				send_period ++;
	    					if (readytosend) { //send_period = data_send_interval)
	      						Schedule -> DeQueue(mdata_sfd ); //send data to max
	      						send_period = 0;
	      						emptyDevDataQueue();           //send device parameters
							readytosend = FALSE;
	    					}
	    					qApp->processEvents(); //force Qt to process events
	  				}
				}
				else { //monitor mode -> loop only stopped by user or negative device status
	  				cout<<"in monitor mode \n";
		  			while (stopflag && status) { //no check for maximal channel number
		    				if (!pause_flag) {	
		      					Schedule->inc_channel(); //increment  channel number
	      						Schedule->reset_integrations();
	      						do {
								status = Schedule->measure(Devices,Priority);
								Schedule->inc_integrations();
							} while (Schedule->act_integrations() <
				      		 			Schedule->maximal_integrations() && status );
							send_period++;
	      						if (readytosend) {
								Schedule -> DeQueue(mdata_sfd); //send data to max
								emptyDevDataQueue();
								send_period=0;
								readytosend = FALSE;
			      				}
			    			}
		  	  			qApp->processEvents(); //force Qt to process  events
	    					if (Schedule->act_channel()==MON_MODE_CHANNELS-1)
	      					Schedule -> set_channel_number(1); //reset channel to 1
			  		}
				}
		
				//close the devices (only if did not already failfail)
				if (init_status && status) {
	  				if (!Schedule -> close_measurement(Devices,Priority))
	    					status = Ierror(E_CLOSING_DEVICES,"closing measurement routine","");
					cout<<"\n****************************************\n";
					cout<<"*          measurement finished !!     *\n";
					cout<<"****************************************\n\n";
				}
				else {
					cout<<"\n***********************************************\n";
			  		cout<<"*  Measurement aborted with ERROR !!!!        *\n";
	  				printf("*  at channel %-5d and integrations : %-5d  *\n",
						Schedule -> act_channel(),Schedule -> act_integrations());
			  		cout<<"************************************************\n";
				}
			}
			emptyDevDataQueue();
			sendDevData(); 			//update database of client
			Schedule -> DeQueue(mdata_sfd); //send data to max in case something is left
			if (sendCommand(STOP,"a",1," ") == 0) { //send stop command!!!
			      Ierror(E_CMD_NOT_SEND,"STOP","");
			      return;
			}
			m_running = 0;
		}
	}

	//internal trigger
  	else {
 		if (!m_running) {       //routine only called once in the beginning
			if (!continue_flag) {
				Schedule -> initialize_SLOT(); //reset measurement parameters
				init_status= Schedule -> init_measurement(Devices,Priority) ;
       				if (init_status)
	 				printf("Polld(internal) : devices initialized\n");
       				else
	 				printf("Error init inifile, quit\n");
      			}	
      
			//setup plot window only if not in high speed mode
      			Schedule -> remove();          //clear database
      			m_running = 1;
      			stopflag  = 1;
      			save_flag = 0;
      			
			if (!Schedule->monitor_mode()) //if not in monitor mode
				continue_flag = 1;           //enable continue button
      			else
				continue_flag = 0;           //make shure that it's zero
      			cout << "starting timer........... \n";
      			//TRUE: timer only timeouts once and then has to be restarted
      			if (init_status) {
				intTrigger -> start(trigger_period,TRUE);
			}
      			else {
	 			printf("\n************************************************\n");
	 			printf("*     Measurement aborted with ERROR !!!!      *\n");
	 			printf("*       in Initialization routines             *\n");
	 			printf("************************************************\n");
	 			emptyDevDataQueue();
	 			sendDevData(); //update database of client
	 			Schedule -> DeQueue(mdata_sfd); //send data to max in case something is left
	 			if (sendCommand(STOP,"a",1," ") == 0) {//send stop command!!!
	   				Ierror(E_CMD_NOT_SEND,"STOP","");
	   				return;
	 			}
	 			m_running = 0;
      			}
   		}
  	}

	//send last channelnumber
	if (sendCommand(CHANNELNUMBER,"dummy",Schedule -> act_channel()," " ) == 0)
		return;
	if (sendCommand(INTEGRATIONS,"dummy",Schedule -> act_integrations(),"" ) == 0)
		return;
}

/*
 *event routine called by timer if internal trigger is selected
 *triggers the measurement routine and passes the arguments
 *-> database adresses
 *has to be called from main, since pointer to devices database and priorities
 *has to be passed to the device functions called from Scheduler
 */
void Polld::TriggerTimerEvent(void) {
  int status=1; //set default value
  if (!pause_flag) { //only measure if PAUSE is not pressed
    status = Schedule->SLOT_measure(Devices,Priority);  //measure
    send_period++;
    if (readytosend) { //send_period = data_send_interval) {
      Schedule -> DeQueue(mdata_sfd); //send data to max
      send_period = 0;
      emptyDevDataQueue();
      readytosend = FALSE;
    }
  }
  //printf("Polld status %d, channel %d, integrations %d \n",
  //	 status,Schedule -> act_channel(),Schedule -> act_integrations());
  if (status && stopflag) { //only restart timer if everything is o.k.
    intTrigger -> start(trigger_period ,TRUE);
    if (monitor && Schedule->act_channel()==MON_MODE_CHANNELS-1){ //monitor mode -> wrap channels
      Schedule -> set_channel_number(1);          //reset channel to 1
    }
  }
  else { //only finish if all integration cycles are finished
    if (!Schedule -> close_measurement(Devices,Priority))
	status = Ierror(E_CLOSING_DEVICES,"closing measurement routine","");
      m_running =0;
      Schedule -> DeQueue(mdata_sfd); //send data to max
      emptyDevDataQueue();
      if (sendCommand(CHANNELNUMBER,"dummy",Schedule -> act_channel(),"" ) == 0)
	return;
      if (sendCommand(INTEGRATIONS,"dummy",Schedule -> act_integrations()," " ) == 0)
	return;
      printf("%smeasurement finished ....\n%s","\033[1m","\033[22m");
      if (sendCommand(STOP,"a",1," ") == 0) {
	Ierror(E_CMD_NOT_SEND,"STOP","");
	return;
      }
  }
}
//Send error command to client and wait in this loop until response comes back
void Polld::SendError(char *message) {
  suspend = TRUE;  //suspend sending of device data while waiting for error response
  sendCommand(ERROR," ",0,message);
  while(wait) {            //wait for answer from user
    QApplication::processEvents();
    usleep(1000);
  }
  wait = 1;
  suspend = FALSE;
}


/*
 * This slot is called, when max puts something in the command socket
 */
void Polld::commandReceived(void)
{
  	command_packet *c_packet = new command_packet;
  	cmdClient_max_len        = sizeof(cmdClient_max);
  	if ((n=recv(cmdr_sfd,( char *) c_packet,CPACKETSIZE,0)) < 0) {
    		CommandNotifier -> setEnabled(FALSE);
    		perror("Polld : command received from ");
    		exit( -1);
  	}
  	memcpy(&cmdvalue,c_packet->data,sizeof(int));
  	switch ( c_packet -> command ) {  //parse command field
  		case START :
    			printf("\nSTART received ....\n");
			if (!m_running) {
				printf("send interval : %d ms max_channels : %d\n",data_send_interval,max_channels);
				stopflag  = TRUE;
				pause_flag = 0;
				continue_flag = FALSE;
				Schedule -> set_max_channel_number(max_channels);
				Schedule -> set_max_integrations(max_integrations);
				Schedule -> reset_channel_number();
				Schedule -> reset_integrations();
				Schedule -> reset_queue(); //clear database
				//      act_channel=act_integrations = 0;
				printf("channels %d,int %d \n",max_channels,max_integrations);
				emit(start());
			}
			break;
		case STOP :
			printf("STOP received...\n");
			stopflag = 0;
   	 		break;
		case PAUSE :
			printf("PAUSE ..... \n");
			if (!pause_flag)
				pause_flag = 1;
			else
				pause_flag = 0;
			printf("pause %d \n",pause_flag);
			break;
		case CONTINUE :
			printf("CONTINUE ....\n");
			if (!m_running && !pause_flag) {
				continue_flag = TRUE;
				stopflag  = TRUE;
				pause_flag = 0;
				Schedule -> reset_integrations();
				Schedule -> reset_queue(); //clear database
				emit(start());
			}
			break;
		case RESET :
			printf("RESET !!! \n");
			break;
		case  CHANNELNUMBER :  //client sets channelnumber
			max_channels = cmdvalue;
			printf("Polld : set channeln to %d\n",max_channels);
			break;
		case  INTEGRATIONS :   //client sets integrations
			max_integrations = cmdvalue;
			printf("Polld : set integrations to %d\n",max_integrations);
			break;
		case  MON_MODE :      //client sets MONITOR MODE
			printf("Polld : monitor - mode enabled\n");
			monitor = 1;
			Schedule -> monitor_mode_on();
			break;
		case  DATAQ_MODE :    //client sets Data Acqusition mode
			printf("Polld  : dataq mode enabled \n");
			Schedule -> monitor_mode_off();
			break;
		case PRIORITY :       //priority added or changed by GUI
			Priority.Pr_append((char *) & ( c_packet -> name ),cmdvalue);
			break;
		case EXT_TRIGGER :
			printf("Polld : switching to external trigger \n");
			Schedule -> set_triggermode(1);
			triggermode = 1;
			break;
		case INT_TRIGGER :
			printf("Polld : switching to internal trigger \n");
			Schedule -> set_triggermode(0);
			triggermode = 0;
			break;
		case SET_PERIOD :
			trigger_period = cmdvalue;
			break;
		case CLEAR_DEVICES :
			printf("Polld : clear devices received\n");
			Devices->remove();
			break;
		case CLEAR_PRIORITY :
			printf("Polld : clear priorities received\n");
			Priority.Pr_remove();
			break;
		case KILL_YOURSELF :
			printf("\n******************************\n");
			printf("Polld : \n");
			printf("Ooops, I got the kill command...\n");
			printf("Good bye\n");
			printf("******************************\n");
			sleep(1);
			//printf("Server is closing connections \n");
			shutdown(data_sfd,2);
			shutdown(mdata_sfd,2);
			shutdown(send_data_sfd,2);
			shutdown(cmd_sfd,2);
			shutdown(cmdr_sfd,2);
			close(data_sfd);
			close(mdata_sfd);
			close(send_data_sfd);
			close(cmd_sfd);
			close(cmdr_sfd);
			QApplication::exit();
			break;
		case ERROR :
			Schedule -> SetErrAnswer(cmdvalue);
			// usleep(100);
			wait = 0;
			break;
		case SET_DAT_SEND_INTERVALL :
			printf("Polld : setting intervall to %d \n",cmdvalue);
			data_send_interval = cmdvalue;
			break;
		case PRIORITY_CHECK :
			if (Priority.N_Devices() != Schedule-> Number_devices()) {
				printf("\n********************************************************************************\n");
				printf("*                  !! ERROR IN YOUR DEVICE-FUNCTION DATABASE !!                *\n");
				printf("* There are %d device names listed in the Priority Initfile                     *\n* But I can only find %d elements in the device_vector in the file 'devices.h'  *\n",Priority.N_Devices() , Schedule-> Number_devices());
				printf("* Max will probably not run with this data if you select the wrong devices !!  *\n* Check your Priority file and the entries of the device vectors in 'devices.h'*\n");
				printf("********************************************************************************\n");
			}
			break;
	}
}
/*
 *send command to client
 *command consists out of command, device name and a variable up
 */
int Polld::sendCommand(char command,char *name,int byte,char *msg)
{
	memset(&c_packet,0,CPACKETSIZE);
	strncpy((char *) c_packet.name,(char *) name,MAX_NAME_LENGTH);
	c_packet.command = command;
	memcpy(&c_packet.data,&byte,sizeof(int));
	strncpy((char *) c_packet.error_msg,(char *) msg,ERR_MSG_LENGTH);
	command_packet *send_c_packet = &c_packet;
	//cout <<"polld : sending cmd to max ...";
	int n;
	if ((n = write(cmd_sfd , send_c_packet,CPACKETSIZE)) < 0) {
		perror("Polld : could not send comand data ");
		return -1;
	}
 	//cout << n << " Bytes send\n";
	return 1;
}
/*
 *send measurement parameters
 *
 */
void Polld::sendData(void){
}
// timer to send data
//only send device Parameters here
//sending of measured data has to be sincronized with the
//measurement routine
//sending is done only after a channel is finished
//controled via the readytosend flag
void Polld::sendParameters(void) {
  if (m_running) {
    if (sendCommand(CHANNELNUMBER,"dummy",Schedule -> act_channel(),"" ) == 0)
      return;
    if (sendCommand(INTEGRATIONS,"dummy",Schedule -> act_integrations(),"" ) == 0)
      return;
    send1 -> start(data_send_interval,TRUE);
    readytosend = TRUE;
    // now also send all device parameter data back to max
    sendDevData();
  }
  return;
}

/*
 *receive device Parameter Data
 *This Slot is called when max sends new device parameters
 *to the polling deamon
 */
void Polld::dataReceived(void) {
    int data_available;
    int i_value=0;
    int enabled;
    double d_value = 0;
    client_len= sizeof(client);

    memset(d_packet,0,DPACKETSIZE);
    while ((data_available = recv(data_sfd,(char *) d_packet,DPACKETSIZE,MSG_PEEK)) > 0  ) {
      if ((n=recv(data_sfd,(char *) d_packet,DPACKETSIZE,0 )) < 0 )
	{
	  DataReceivedNotifier -> setEnabled(FALSE);
	  perror("Error : server received from ");
	  exit(-1);
	}
      else {
	    if (!m_running) {   //insert Element into local device-parameter database
		//test for variable type : 1 =int,2=double,4 = string
		//copy endbaled flag
		enabled = (int ) d_packet -> enabled;
		if (d_packet->type == '1' ) {
		    i_value = strtol((const char * ) d_packet -> value,NULL,10);
		    Devices->append((int) i_value,(char *)d_packet -> device_name,
				    (char *)d_packet -> var_name,enabled,1,0,0);
		}
		else if (d_packet ->type == '2') {
		    d_value = strtod((const char *) d_packet -> value,NULL);
		    Devices->append((double) d_value,(char *)d_packet -> device_name,
				    (char *)d_packet -> var_name,enabled,2,0,0);
		}
		else {
		    Devices->append(0,(char *)d_packet->device_name,"active",0,1,0,-1);
		}
	    }
	    else {//measurement running -> stack packets
		DevRQueue -> append(d_packet);
	    }
	}
    }
}
/*
 *empty the list of received and stacked device data :
 *this routine is called in between two channels
 *to update the local device database
 */

void Polld::emptyDevDataQueue(void) {
  devDataPacket * pointer;
  int    i_value;
  double d_value;

  for (pointer = DevRQueue -> first(); pointer != NULL ;
       pointer = DevRQueue -> next()) {
  if (pointer -> type == '1' ) {
    i_value = strtol((const char *) pointer -> value,NULL,10);
    printf("value %d \n",i_value);
    Devices->append((int) i_value,(char *) pointer -> device_name,
		   (char *) pointer -> var_name,0,1,0,0);
        }
    if (pointer ->type == '2') {
      d_value = strtod((const char *) pointer -> value,NULL);
      printf("value %f \n",d_value);
      Devices->append((double) d_value,(char *)pointer -> device_name,
		     (char *)pointer -> var_name,0,2,0,0);
    }
  }
  DevRQueue -> clear();
}
/*
 *This routine sends all devicedata
 *out of sendqueue to the client
 *the sendqueue is filled each time the device modifies
 *an element of the device database
 */
void Polld::sendDevData(void ) {
  if ( ! suspend ) {  //Only send device data if not suspended by error message
    devDataPacket * sendPacket;
    int endoflist = 1;
    Devices->ResetHelpPointer();
    while(endoflist) {
      sendPacket = Devices->GetNextPacket();
      endoflist  = Devices->AdvanceHelpPointer();
      if (send(send_data_sfd ,(char *)  sendPacket,DPACKETSIZE,0) <0 ) {
	perror("polld : could not send device data ");
	return;		
      }
    }	
  }
}
/*
 * returns sendPacket out of devname, var_name and value
 * used for sending device parameter data to max
 */
devDataPacket * Polld::makeDevPacket(char *devname,char *varname,int i_var ) {
  devDataPacket *Packet = new devDataPacket;
  assert (Packet !=0 );
  char *buffer;
  buffer = new char[MAX_NAME_LENGTH+MAX_VAR_LENGTH];
  strncpy((char *) Packet ->device_name,
	  devname,MAX_NAME_LENGTH);
  strncpy((char *) Packet -> var_name,
	  varname,MAX_NAME_LENGTH);
  //int
  sprintf( buffer," %d",i_var);
  strncpy(( char *) Packet -> value, buffer,MAX_VAR_LENGTH);
  Packet -> type = '1';
  return Packet;
}
devDataPacket * Polld::makeDevPacket(char *devname,char *varname,double d_var) {
  devDataPacket *Packet = new devDataPacket;
  assert (Packet !=0 );
  char *buffer;
  buffer = new char[MAX_NAME_LENGTH+MAX_VAR_LENGTH];
  strncpy((char *) Packet ->device_name,
	  devname,MAX_NAME_LENGTH);
  strncpy((char *) Packet -> var_name,
	  varname,MAX_NAME_LENGTH);
  //double
  sprintf( buffer,"%G", d_var);
  strncpy(( char *) Packet -> value, buffer,MAX_VAR_LENGTH);
  Packet -> type = '2';//(char ) BufferPointer -> type;
  return Packet;
}
/**************************************************
 * Network stuff
 * the following routines establish the network connection
 * via sockets
 * here you also find the SLOTS that are called  via QSocketNotifer
 * send routines are here as well
 * The ports to connect on have to be known to both processes
 ***************************************************/
int Polld::socketConnect(char *hostname) {
	printf("Polld : Polld::socketConnect(%s)\n",hostname);
  	
	hostname = 0;
  	int	sfd_1 = 0; //socket filedescriptors
  	int	sfd_2 = 0;
  	int 	sfd_3 = 0;
  	int 	sfd_4 = 0;
  	int	sfd_5 = 0;

  	struct hostent *host;
  	struct in_addr in;

  	if (!(host = gethostbyname("localhost"))) { //actually not needed
    		perror("Polld : Could not get hostname for client \n");
    		return -1;
  	}
  	memset(&server_addr,0,sizeof(server_addr));
  
	while ( *host ->h_addr_list ) {
    		memcpy(&in.s_addr,*host -> h_addr_list,sizeof(in.s_addr));
    		++host -> h_addr_list;
  	}
  	server_addr.sin_family          = AF_INET;
  	server_addr.sin_addr.s_addr     = 127 + 0x1000000;// hotnl(INADDR_ANY)
  	server_addr.sin_port            = htons( DEFAULT_PORT+4 );
  
	// Create one socket which accepts all requests from the client
#ifdef DEBUG_POLLD
	printf("Polld : Creating socket\n");
  	printf("Polld using port to send  Data  : %d \n",ntohs(server_addr.sin_port) );
  	printf("Polld using adress to send Data : %s \n",inet_ntoa(server_addr.sin_addr) );
#endif
	
	if ((main_sfd=socket(AF_INET, SOCK_STREAM,0)) < 0) {
		perror("Server could not create the socket, giving up !!!\n");
		return -1;
	}
  //enable reuse of sockets still marked as open in the kernel
  int on = 1;
  setsockopt(main_sfd, SOL_SOCKET, SO_REUSEADDR,(const char *)  &on, sizeof (on));
 
  if (bind(main_sfd, (struct sockaddr * ) &server_addr,
	   sizeof(server_addr) ) < 0) {
    perror("Polld : Could not bind main_sfd\n");
    return -1;
  }

  if (listen(main_sfd,5) < 0) {
      perror("Polld : listen for dataSend Connection\n");
      close(main_sfd);
      return -1;
  }

  //now accept five socket connections to the one client
  //the client has to connect in exactly this order
  data_send_len = sizeof(data_send);
  if ((sfd_1 = accept(main_sfd,(struct sockaddr *)&data_send,
		       (unsigned int *) &data_send_len)) < 0   ) {
      perror("Polld : accept send Data socket");
      close(main_sfd);
      return -1;
  }	

  // connect comand send socket (polld is "server")
  memset(&cmd_server_pd_addr,0,sizeof(struct sockaddr_in));
  memset(&cmd_client_max_addr,0,sizeof(struct sockaddr_in));
  cmd_server_pd_addr.sin_family          = AF_INET;
  cmd_server_pd_addr.sin_addr.s_addr     = htonl(INADDR_ANY);
  cmd_server_pd_addr.sin_port            = htons( DEFAULT_PORT+3 );
  cmd_server_pd_addr_len = sizeof( cmd_server_pd_addr);

  if ((cmd_main_sfd=socket(AF_INET, SOCK_STREAM,0)) < 0) {
    perror("Server could not create socket for sending cmd, giving up\n");
    return -1;
  }
  on = 1;
  setsockopt(cmd_main_sfd, SOL_SOCKET, SO_REUSEADDR,(const char *)  &on, sizeof on);

  if (bind(cmd_main_sfd, (struct sockaddr * ) &cmd_server_pd_addr,
	   sizeof(cmd_server_addr)) < 0) {
    perror("Polld : Could not execute bind to bind cmd_sfd \n");
    return -1;
  }
  if (listen(cmd_main_sfd,1)) {
      perror("Polld : listen for dataSend Connection");
      close(cmd_main_sfd);
      return -1;
  }
  if ((sfd_2 = accept(cmd_main_sfd,(struct sockaddr *)&cmd_client_max_addr,
		      (unsigned int *) &cmd_client_max_addr_len)) < 0   ) {
    perror("Polld : failed accept Command receive socket");
    close(main_sfd);
    return -1;
  }	
  /*
   *Connect Receive command sockets (polld is "client")
   */
  cmd_client_pd_addr.sin_family          = AF_INET;
  cmd_client_pd_addr.sin_addr.s_addr     = htonl(INADDR_ANY);
  cmd_client_pd_addr.sin_port            = htons(DEFAULT_PORT+2 );
  cmd_client_pd_addr_len = sizeof( cmd_client_pd_addr);
  if ((cmd_main_sfd2 =socket(AF_INET, SOCK_STREAM,0)) < 0) {
    perror("Server could not create socket for receving cmd, giving up\n");
    return -1;
  }
  on = 1;
  setsockopt(cmd_main_sfd2, SOL_SOCKET, SO_REUSEADDR,(const char *)  &on, sizeof on);
  //bind DevDataSendsocket  to adress
  if (bind(cmd_main_sfd2, (struct sockaddr * ) &cmd_client_pd_addr,
	   sizeof(cmd_client_addr)) < 0) {
    perror("Polld : Could not execute bind to bind cmdr_sfd\n");
    return -1;
  }
  if (listen(cmd_main_sfd2,1) < 0) {
      perror("Polld : listen for dataSend Connection");
      close(cmd_main_sfd2);
      return -1;
  }
  if ((sfd_3 = accept(cmd_main_sfd2,(struct sockaddr *)&cmd_server_max_addr,
		      (unsigned int *) &cmd_server_max_addr_len)) < 0   ) {
    perror("Polld : accept cmd_send socket");
    close(main_sfd);
    return -1;
  }

   // Connect Send Parameters Socket (Polld acts as  "server")
  memset(&par_server_pd_addr,0,sizeof(struct sockaddr_in));
  memset(&par_client_max_addr,0,sizeof(struct sockaddr_in));
  par_server_pd_addr.sin_family          = AF_INET;
  par_server_pd_addr.sin_addr.s_addr     = htonl(INADDR_ANY);
  par_server_pd_addr.sin_port            = htons( DEFAULT_PORT+1 ); //use send port -> see definitions.hpp
  par_server_pd_addr_len = sizeof( par_server_pd_addr_len);
  if ((par_main_sfd=socket(AF_INET, SOCK_STREAM,0)) < 0) {
    perror("Server could not create socket for sending devData to client ,
            giving up !!!\n");
    return -1;
  }
  on = 1;
  setsockopt(par_main_sfd, SOL_SOCKET, SO_REUSEADDR,(const char *)  &on, sizeof on);
  if (bind(par_main_sfd, (struct sockaddr * ) &par_server_pd_addr,
	   sizeof(par_server_pd_addr)) < 0) {
    perror("Polld : Could not execute bind to bind sendDevData socket\n");
    return -1;
  }
  if (listen(par_main_sfd,1)) {
      perror("Polld : listen for DevDataSend Connection");
      close(par_main_sfd);
      return -1;
  }
  if ((sfd_4 = accept(par_main_sfd,(struct sockaddr *)&par_client_max_addr,
		      (unsigned int *) &par_client_max_addr_len)) < 0   ) {
    perror("Polld : accept parameters send socket");
    close(par_main_sfd);
    return -1;
  }	
  /*
   *Connect Device Parameters receive Socket  (polld acts as client)
   */
  memset(&par_client_pd_addr,0,sizeof(struct sockaddr_in));
  memset(&par_server_max_addr,0,sizeof(struct sockaddr_in));
  par_client_pd_addr.sin_family      = AF_INET;
  par_client_pd_addr.sin_addr.s_addr = htonl(INADDR_ANY);   //serve to any adress
  par_client_pd_addr.sin_port        = htons(DEFAULT_PORT); // proper byte ordering

  if ((par_main_sfd2 =socket(AF_INET, SOCK_STREAM,0)) < 0) {
    perror("Server could not create socket, giving up !!!\n");
    exit(1);
  }
  on = 1;
  setsockopt(par_main_sfd2, SOL_SOCKET, SO_REUSEADDR,(const char *)  &on, sizeof on);
  par_client_pd_addr_len = sizeof(par_client_pd_addr);
  if (bind(par_main_sfd2,(struct sockaddr *) &par_client_pd_addr,
	   par_client_pd_addr_len) < 0 ) {
    perror ("DevData receive :  Server bind error");
    return -1;
  }

  if (listen(par_main_sfd2,1) < 0) {
      perror("Polld : listen for devData receive Connection");
      close(par_main_sfd2);
      return -1;
  }
  if ((sfd_5 = accept(par_main_sfd2,(struct sockaddr *)&par_server_max_addr,
		      (unsigned int *) &par_server_max_addr_len)) < 0   ) {
    perror("Polld : accept parameters receive socket");
    close(main_sfd);
    return -1;
  }	
  //set the receive sockets sfd_2 and sfd_5 to NON BLOCKING status
  unsigned long asyncOn = 1;
  if (fcntl(sfd_2, F_SETFL, O_NONBLOCK) < 0) {
      perror("S:fcntl Command Receive socket ");
      close (sfd_2);
      return -1;
  }
  asyncOn = 1;
  if (ioctl(sfd_5, FIONBIO,&asyncOn) < 0) {
      perror("S:ioctl Command Receive socket ");
      close (sfd_5);
      return -1;
  }

#ifdef DEBUG_POLLD
  printf("Polld : 5 sockets successfully accepted, I will continue ...\n");
  printf("Polld : filedescriptors are : %d %d %d %d %d\n",
		sfd_1,sfd_2,sfd_3,sfd_4,sfd_5);
#endif
  
  // Copy filedescriptors to old names used in the program
  mdata_sfd 	= sfd_1;
  cmd_sfd   	= sfd_2;
  cmdr_sfd  	= sfd_3;
  send_data_sfd = sfd_4;
  data_sfd    	= sfd_5;
  return 1;
}


/********************************************************************
 *
 *  Main programm  :
 *
 *******************************************************************/

/* Signal handler
 * to catch interrupt signals initiated by the user
 * This way  the server can only be killed
 * via the Main GUI's quit command
 */
void signalhandler( int signum ) {
  switch(signum) {
  case SIGINT:
    printf("SIGINT caught \n");
    break;
  case SIGTERM:
    printf("SIGTERM caught \n");
    break;
  case SIGABRT:
    printf("SIGABRT caught \n");
    break;
  case SIGPIPE:
    printf("SIGPIPE caught \n");
    exit(-1);
    break;
  case SIGSEGV:
    printf("SIGSEV caught \n");
    exit(-1);
    break;
  case SIGTSTP:
    printf("SIGTSTP caught \n");
    break;
  case SIGQUIT:
    printf("SIGQUIT caught \n");
    break;
  default :
    break;
    printf("Please terminate the polld via the Main Window (Quit) ! \n");
  }
}

int main(int argc,char *argv[]) {
  /* Connect signals for the signalhandler
   * It catches signals like SIGSEGV and SIGQUIT no  be able to notify the 
   * client if the deamon dies, so that the sockets can be unbound and closed 
   * properly
   * This has to be done outside the class otherwise the signal is not cought properly
   */
	if (strcmp(argv[0],"CHILD") != 0) {
		printf("Polld : Polling Deamon running whithout parent\n");
		/*    signal(SIGINT,&signalhandler);
    		signal(SIGTERM,&signalhandler);
    		signal(SIGABRT,&signalhandler);
    		signal(SIGPIPE,&signalhandler);
    		signal(SIGSEGV,&signalhandler);
    		signal(SIGTSTP,&signalhandler);
    		signal(SIGQUIT,&signalhandler);*/
  	}
  	else
    		printf("Polld : Polling Deamon running as child of Max\n");

  	//Pass the five socket filedescriptors as arguments to the polld class

  	Polld *pD = new Polld(argc,argv);
  	CHECK_PTR(pD);
  	return pD ->exec(); //pass control to Qt
}
