/*
 * MAX : sched.cc, Sam Dez 29 20:06:37 CET 2001 -sg
 *
 * this file contains the implementation of the priority_list and the 
 * scheduler class examine header file to see all functions accessible 
 * by the device functions
 *
 * This file is part of Max data acquisition software
 * Copyright (C) 1998 Christian Rosen
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <iostream.h>
#include <stdarg.h>
#include <qapp.h>
#include <qimage.h>

#include "devices.h"
#include "sched.hh"
#include "list_polld.h"
#include "error.hh"
#include "data.h"

#undef DEBUG
//#define DEBUG
/**
  *Trigger function calls internal timer - waits then for timeout
  *remove this function to seperate file !!!!!!!!!!!!!!!!!!!!!!
  *return 1 if trigger recieved
  */
void Trigger() {
  //printf("wait for trigger \n");
}

//*****************************************************************************
//
//here comes the implementation of the scheduler class
//this class performs the easurement
//and does the device function calls via a list of pointers to functions
//(Function_list)
//
//****************************************************************************/
Scheduler::Scheduler(int a, int b, int c, int d) {
  wait             = 1;
  err_answer       = 1;
  max_channels     = a;
  max_integrations = b;
  s_mode           = c;
  t_mode           = d;  //scheduler variables
  t_mode           = 1;
  monitor          = 0; //monitor mode off
  buffPtr  = new backup[ACTIVE_VARIABLES];
  assert(buffPtr !=0);
  arrayPtr = new arraybackup[ACTIVE_VARIABLES];
  assert(arrayPtr !=0);
  backup_array=new int[BACKUP_ARRAY_SIZE];
  assert(backup_array !=0 );
  period = 0;  //initialize trigger delay "period" to 0ms
  Que = new PacketList;
}
//destructor
Scheduler::~Scheduler() {
  remove(); //kill all databases
  delete Que;
  delete buffPtr;
  delete backup_array;
  delete arrayPtr;
}
/*this is the error function for individual error messages
 *use this if you want to print your own text in the
 *error window
 */
int Scheduler::error(char *error,char *formatstr, ... ) {
  char     *msg = new char [ERR_MSG_LENGTH];
  char     *qtmsg = new char [ERR_MSG_LENGTH+50];
  va_list  args;

  va_start(args,formatstr);
  vsprintf(msg,formatstr,args);
  va_end(args);
  sprintf(qtmsg,"%s%s",error,msg);
  emit(Sig_Error(qtmsg));
  return err_answer;
}

//routine calls all function in the initdevices vector in decies.h one time
//an returns. This routine is called at the beginning of the measurement
int Scheduler::init_measurement(List *Device_list,Pr_list &Priority_list)
{
  maxarraysize = 1;
  priority    = 1;      //priority index
  int counter = 0;      // devices index
  status  = 1;      //return-status of devices FALSE stops initialization
  while (priority <= NUMBER_DEVICES && status ) {
    counter = 0;
    while (counter < NUMBER_DEVICES && status) {
      if (Priority_list.Get_priority((int) counter) == priority)
	status = (*Init_Function_List[counter])(Device_list, this);
      counter ++;          //next device
    }
    priority ++;          //next device
  }
return status;
}
//calll\ed at the end of the measurement procedure
//to close all devices
int Scheduler::close_measurement(List *Device_list,Pr_list &Priority_list)
{
  priority    = 1;      //priority index
  int counter = 0;      // devices index
  close_status  = 1;      //return status of defices FALSE stops initialization
  while (priority <= NUMBER_DEVICES && status ) {
    counter = 0;
    while (counter < NUMBER_DEVICES && status) {
      if (Priority_list.Get_priority((int) counter) == priority)
	close_status = (*Close_Function_List[counter])(Device_list, this);
      counter ++;          //next device
    }
    priority ++;          //next device
  }
return close_status;
}
//this routine performs the measurement but returns after one cycle
//of priorities
//this is neccessary to check the status of the stop button in the main window
//this functions is called until channel number = max_channels
//or STOP is pressed
//printf("channel, integrations, counter, priority %d %d %d %d \n",channels
//,integrations,counter,priority);	
 //printf("channel, integrations, counter, priority %d %d %d %d \n",channels,
//integrations,counter,priority);
int Scheduler::measure(List *Device_list, Pr_list &Priority_list)
{
  int counter;
  status = 1;
  stop =1 ;
  //
  //Start loop over channels and priorities
  //leave trigger routine empty if you trigger by first device
  Trigger();         //First wait for external trigger
  //
  //pretrigger loop
  //
  priority = 1;
  while (priority <= NUMBER_DEVICES && status) {
    counter = 0;
    while (counter < NUMBER_DEVICES && status) {
      qApp -> processEvents();
      if (Priority_list.Get_priority((int) counter) == priority) {
	if (!integrations) {  //during first cycle, all devices are called
	  status = (*Pre_Function_List[counter])(Device_list, this);
	}
	else    //After first cycle, only call active devices
	  if (Device_list->is_active(Priority_list.Get_device_name(counter)) ==1 ) {
	    qApp -> processEvents();
	    status = (*Pre_Function_List[counter])(Device_list, this);  //Call Device_Function of "active" devices only
	  }
      }
      counter ++;
    }
    priority ++;
  }
  //end of pretrigger loop
  //
  //main loop
  //
  priority = 1;
  while (priority <= NUMBER_DEVICES && status) {
    counter = 0;
    while (counter < NUMBER_DEVICES && status) {
      if (Priority_list.Get_priority((int) counter) == priority) {
	if (!integrations) {  //during first cycle, all devices are called
	  status = (*Function_List[counter])(Device_list, this);
	}
	else    //After first cycle, only call active devices
	  if (Device_list->is_active(Priority_list.Get_device_name(counter)) ==1 ) {
	    status = (*Function_List[counter])(Device_list, this);
	  }
      }
      counter ++;
    }
    priority ++;
  }
  //
  //end of main loop
return status;
}
/*SLOT_measure :
 *event controlled measurement routine
 *called by timer or other external event routine
 *avoids qApp->processEvents
 *and is more elegant anyway
 *call routine by timer -> timer start set by main routine after having called the data_initroutine
 *data_init resets actual channel number and so forth
 *user trigger routine then does hardware polling and emits the signal or is represented by an device
 *initialize resets all parameters
 *only call this routine to initialize interrupt controlled measurement
 */
void Scheduler::initialize_SLOT( void)
{
  busy             = FALSE; //no measurement running
  channels         = 0;
  integrations     = 0;
  stop             = 1;
  device_status    = 1;
  maxarraysize     = 1;

  Que -> remove();
}
//
// this function is called from
// void MainMenu::TriggerTimerEvent(void)
//
int Scheduler::SLOT_measure(List *Device_list,Pr_list &Priority_list)
{
 int counter;
 int chancopy;
 device_status = 1;
 // printf("SLOT_measure called channel %d int %d\n",channels,integrations);
 if (!busy) {
   busy = TRUE; //block all other measurement events
   if (!monitor)
     chancopy = channels; //not in monitor mode -> count channels
   else
     chancopy =0; //disable channel-number stop criteria
   if (chancopy < max_channels) {
     channels++;
     //     printf("\nchancopy, channels, max_channels monitor int max_int  %d, %d %d, %d, %d %d\n",
     //	 chancopy,channels,max_channels,monitor,integrations,max_integrations);
     stop     = 1;
     priority = 1;
     //
     //pretrigger loop
     //
     while (priority <= NUMBER_DEVICES && device_status) {//inner loop is always identical
       counter = 0;
       while (counter < NUMBER_DEVICES && device_status) {
	 if (Priority_list.Get_priority((int) counter) == priority) {
	   if (!integrations) {  //during first cycle, all devices are called
	     device_status = (*Pre_Function_List[counter])(Device_list, this);
	   }
	   else                  //After first cycle, only call active devices
	     if (Device_list->is_active(Priority_list.Get_device_name(counter)) ==1 ) {
	       device_status = (*Pre_Function_List[counter])(Device_list, this);
	     }
	 }
	 counter ++;
       }
       priority ++;
     }
     priority = 1;
     device_status=1;
     //
     //end of pretrigger loop
     //
     /*inner loop*/
     while (priority <= NUMBER_DEVICES && device_status) { //inner loop is always identical
       counter = 0;
       while (counter < NUMBER_DEVICES && device_status) {
	 if (Priority_list.Get_priority((int) counter) == priority) {
	   if (!integrations) {  //during first cycle, all devices are called
	     device_status = (*Function_List[counter])(Device_list, this);
	   }
	   else                  //After first cycle, only call active devices
	     if (Device_list->is_active(Priority_list.Get_device_name(counter)) ==1 ) {
	       device_status = (*Function_List[counter])(Device_list, this);
	     }
	 }
	 counter ++;
       }
       priority ++;
     }
     channels--;
     //integrations++;     /*end inner loop*/
     if (++integrations == max_integrations) {
       integrations = 0;
       channels ++;
     }
     busy = FALSE;
   }
   else {
     printf("measurement finished \n");
     printf("switching off timer now\n");
     device_status = 0;
   }
 }
 return device_status; //if device_status < 1 measurement stops
}
int  Scheduler::trigger_mode          (void ) { return t_mode    ; }
int  Scheduler::maximal_integrations  (void ) { return max_integrations; }
int  Scheduler::act_integrations      (void ) { return integrations; }
int  Scheduler::maximal_channels      (void ) { return max_channels; }
int  Scheduler::act_channel           (void ) { return channels;     }
int  Scheduler::monitor_mode          (void ) { return monitor;      }
uint Scheduler::delay                 (void ) { return period ;      }
void Scheduler::set_max_channel_number(int x) { if (x) max_channels = x ; }
void Scheduler::set_max_integrations  (int x)
{
  if (x && x<MAX_INTEGRATIONS)
    max_integrations = x ;
}
void Scheduler::set_channel_number    (int x) { channels = x     ;   }
void Scheduler::set_cmd_sfd           (int x) { cmd_sfd = x; }
void Scheduler::SetWaitFlag     (int x,int y) { err_wait = x; err_answer = y;}
void Scheduler::SetErrAnswer          (int x) { err_answer = x;}
void Scheduler::set_triggermode       (int x)
{
  if ( x<2 && x> -1 )
    t_mode = x;
}
void Scheduler::inc_integrations      (void ) { integrations++   ; }
void Scheduler::inc_channel           (void ) { channels++       ; }
void Scheduler::reset_channel_number  (void ) { channels = 0     ; }
void Scheduler::reset_integrations    (void ) { integrations = 0 ; }
void Scheduler::reset_queue           (void ) { Que -> remove()   ; }
void Scheduler::reset                 (void )
{
  integrations = 0;
  channels     = 0;
  Que -> remove();
}
void Scheduler::external_trigger      (void ) { t_mode = 1       ; }
void Scheduler::internal_trigger      (void ) { t_mode = 0       ; }
void Scheduler::monitor_mode_on       (void ) { monitor = 1      ; }
void Scheduler::monitor_mode_off      (void ) { monitor = 0      ; }
int Scheduler::Number_devices              (void ) { return NUMBER_DEVICES; }
//
//here are the SLOTS of the scheduler used to control the measurement
//from the Qt_application
//
void Scheduler::stop_measurement(int a)
{
  a = 0;
  channels = max_channels;
}
void Scheduler::SLOT_setdelay(uint x)
{
  period = x;
  //printf("setting trigger period to %d\n",period);
}
/*
 *Trigger Slot called by timeout of trigger
 *Scheduler waits until wait = TRUE
 *and then continues with the measurement
 *modify (jump into) this routine, when you want to have interupt controlled measurement
*/
void Scheduler::SLOT_timeout(void ) {
  wait = TRUE;
}

//remove complete measurement data_list out of memory
void Scheduler::remove() {
  temp_counter       = 0; //reset counters for store routine
  element_counter    = 0;
  var_index          = 0; //index for plot variables
  temp_var_index     = 0;
  array_counter      = 0;
  array_temp_counter = 0;
  maxarraysize       = 0;
  for (int i=0;i< ACTIVE_VARIABLES; i++) {
    buffPtr[i].i_var = 0;
    buffPtr[i].d_var = 0;
    buffPtr[i].type  = 0;
    if (arrayPtr[i].iptr ) {     //if array exists at index i
      printf("deleting arry\n");
      delete [] arrayPtr[i].iptr; //free memory there
    arrayPtr[i].size = 0;
    }
  }

}

void Scheduler::remove_continue() {
  temp_counter       = 0; //reset counters for store routine
  element_counter    = 0;
  temp_var_index     = 0;
  array_counter      = 0;
  array_temp_counter = 0;
  maxarraysize       = 0;
  for (int i=0;i< ACTIVE_VARIABLES; i++) {
    buffPtr[i].i_var = 0;
    buffPtr[i].d_var = 0;
    buffPtr[i].type  = 0;
    if (arrayPtr[i].iptr ) {     //if array exists at index i
      printf("deleting arry\n");
      delete [] arrayPtr[i].iptr; //free memory there
    arrayPtr[i].size = 0;
    }
  }
}

//
//this routine is caled by the user
//store routine stores measured  user data in database
//performs integratins (via buffer array BufPtr)
//and sets typelist of variables for save routine
//in the array of struct buffPtr[i]
//that is limited to 1000 var's so far -> increase it in sched.h
//
void Scheduler::store(int data)
{
  /*
   *during first channel and first integrations cycle
   *count number of active vars
   *in element_counter only store data if not in monitor mode
   *if int=1 -> data has to be stored right away with add-data!
   */
  if (channels == 1 && !integrations) {
    buffPtr[element_counter].i_var = data; //fill bufPtr vector with measured data
    buffPtr[element_counter++].type = 1; //Set type info for save routine
    var_index++;
    if (max_integrations==1/* && !monitor*/)
     Que-> append((int) data,channels,priority,var_index-1);
    return;
  }
  if (integrations < max_integrations-1) { //during following cycle add data to buffPtr
    if (!integrations)
      buffPtr[temp_counter].i_var = data;
    else
      buffPtr[temp_counter].i_var += data;
    if (temp_counter++ == element_counter-1) {
      temp_counter = 0;
    }
    temp_var_index++;
    if (temp_var_index == var_index)
      temp_var_index =0;
    return;
  }
  /***************only store if not in monitor mode ****/
  if (max_integrations==1)
    buffPtr[temp_counter].i_var=0;
  //  if (/*!monitor*/ ) //value,priority,variable_index
    Que -> append(data+buffPtr[temp_counter].i_var,channels,priority,temp_var_index);
    temp_counter++;
    temp_var_index++;
    if (temp_counter == element_counter)
      temp_counter = 0;     //reset counter after one cycle of devices
    if (temp_var_index == var_index)
      temp_var_index =0;
}

void Scheduler::store(double data)
{
  //
  //during first channel and first integrations cycle count number of active vars
  //
  if (channels==1 && !integrations) {
    buffPtr[element_counter].d_var = data; //fill buffPtr vector with measured data
    buffPtr[element_counter++].type = 2; //Set type info for save routine
    var_index ++;
    if (max_integrations==1 /*&& !monitor*/)
      Que-> append(data,channels,priority,var_index-1); //queue data
    return;
  }
  if (integrations < max_integrations-1) { //during following cycle add data to buffPtr
    if (!integrations)
      buffPtr[temp_counter].d_var = data;
    else
      buffPtr[temp_counter].d_var += data;
    if (temp_counter++ == element_counter-1)
      temp_counter = 0;
    temp_var_index++;
    if (temp_var_index == var_index)
      temp_var_index =0;
    return;
  }
  if (max_integrations==1)
    buffPtr[temp_counter].d_var=0;
  /***************only store if not in monitor mode ****/
  //if (1/*!monitor*/) //value,priority,variable_index
     Que -> append(data+buffPtr[temp_counter].d_var,channels,priority,temp_var_index);
  temp_counter++;
  if (temp_counter == element_counter)
    temp_counter = 0;             //reset counter after one cycle of devices
  temp_var_index++;
  if (temp_var_index == var_index)
    temp_var_index =0;
}
/*******************************************************************
store routine for integer arrays
array passed by reference -> copy neccessary

element_counter contains total number of stored variables per cycle
determined at the beginning of the measurement (channel == 0,
integrations ==)
temp_counter used for integration of element_counter store variables
the size of the array is determined during first cycle (channel ==0,
integrations ==0)
increase of array size after first channel is not accepted anymore !!!
*****************************************************************/
void Scheduler::store( int array[],int size)
{
  int x;
  if (size > MAX_ARRAY_SIZE)                        //truncate array to maximal size
    size = MAX_ARRAY_SIZE;                          //if neccessary
  if (size > maxarraysize)
    maxarraysize = size;                            //get size of largest array
  if (channels==1 && !integrations) {                //count number of arrays per channel and alocate the mem
    arrayPtr[array_counter].iptr = new int[size+1];  //reserve space in array
    assert (  arrayPtr[array_counter].iptr != 0);    //check for memory leak
    arrayPtr[array_counter].size = size;             //copy size of this string
    for (x = 0;x < size;x++)
      arrayPtr[array_counter].iptr[x] = array[x];     //copy array into buffer
    array_counter ++;                                 //array_counter = # of arrays
    var_index++;
    if (max_integrations == 1)
      Que -> append(arrayPtr[array_counter-1].iptr,arrayPtr[array_counter-1].size,
		    channels,priority,var_index-1);
    return;
  }
  if (integrations < max_integrations-1) {    //during following cycle add data to buffPtr
    if (!integrations)
      for (x = 0;x < arrayPtr[array_temp_counter].size;x++)  //only copy size determined in first cycle
	arrayPtr[array_temp_counter].iptr[x] = array[x]; //copy array into buffer
    else
      for (x = 0;x < arrayPtr[array_temp_counter].size;x++)
	arrayPtr[array_temp_counter].iptr[x] += array[x]; //add array to buffer
    temp_var_index ++;
    if (array_temp_counter++ == array_counter-1)
      array_temp_counter = 0;                   //reset counter after one cycle of devices
    if (temp_var_index ==  var_index)
      temp_var_index =0;
    return;
  }
  // here the channel is finished and the data is stored away in the sendbuffer
  if (max_integrations==1)                 //for one integration  -> copy array because
    for (x = 0;x <  arrayPtr[array_temp_counter].size;x++)  //array buffer is not cleared after measurement
      arrayPtr[array_temp_counter].iptr[x]  = array[x];     //add array to buffer
  else
    for (x = 0;x <  arrayPtr[array_temp_counter].size;x++)
      arrayPtr[array_temp_counter].iptr[x] += array[x];     //add array to buffer
  Que -> append(arrayPtr[array_temp_counter].iptr,arrayPtr[array_temp_counter].size,
  	channels,priority,temp_var_index);       //store dat
  array_temp_counter++;
  temp_var_index++;
  if (array_temp_counter == array_counter)
    array_temp_counter = 0;
  if (temp_var_index == var_index)
    temp_var_index =0;
}
//
//this store routine reads an Image from a filename given and plots it on the local screen
//
void Scheduler::store( char *fname) {
  QImage bufferImage;
  if (!integrations) {    //no integrations of images, only the first one is stored !!
    bufferImage.load(fname,0); //load image from disk into Qimage
    //    add_data(&bufferImage);
    printf("no image processing yet\n");
    emit(send_data(&bufferImage,priority));
    //  qApp -> processEvents();
  }
}

//
//DeQueue the list of dataPackets that contain the measured data
//and send them via sendto via socket given in arguments
//
int Scheduler::DeQueue(int sd)
{
  int x=0;
  bool f = TRUE;
  PacketElement *P; //temporary pointer
   for (P = Que -> first();P != 0;P = Que -> next()) {
    x++;
    if (send(sd , (char *) Que -> packet(),FULL_P_SIZE,0) < 0) {
      perror("Sched::DeQueue : could not send data ");
      }
    f = FALSE;
  }
  Que -> remove(); //erase queue for next run
  delete P;
  return 1;
}

