/*
 * MAX : utils.cc, Sam Dez 29 20:10:16 CET 2001 -sg
 *
 * main file for utilities
 * here you find some small widgets t oset options and the save widget
 * they all belong to max.c, the main GUI
 *
 * This file is part of Max data acquisition software
 * Copyright (C) 1997,98 Christian Rosen
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "definitions.h"
#include "qtincludes.h"
#include "utils.h"
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <math.h>
#include <string.h>
#define XSize 200
#define YSize 150
/*
 *Small widget to enable/disable store variables (e.g. "channels")
 *of a device
 *also allows to select/deselect the variable for plotting
 */
Options::Options(const char *dname,int save,int p)
{
  printf("en %d plot %d \n",save,p);
  enable = save;
  plot =p;
  setCaption(dname);
  ok        = new QPushButton(this,"ok");
  cancel    = new QPushButton(this,"cancel");
  ok       -> setText("o.k.");
  cancel   -> setText("cancel");
  connect(ok,SIGNAL(clicked()),SLOT(ok_pressed()));
  connect(cancel,SIGNAL(clicked()),SLOT(cancel_pressed()));
  setFixedSize(XSize,YSize);//window has fixed size -> small
  ok        -> setGeometry(25,100,50,30);
  cancel    -> setGeometry(125,100,50,30);

  //create a group of check boxes

  QButtonGroup *bg = new QButtonGroup( this, "bla");
  bg->setTitle( "Select Options" );
  cb[0] = new QCheckBox( bg );
  cb[0]->setText( "Enable Channel" );
  cb[0]->setGeometry( 10,15, 100,25 );
  cb[1] = new QCheckBox( bg );
  cb[1]->setText( "Plot It " );
  cb[1]->setGeometry( 10,45, 100,25 );
  bg->setGeometry( 10,10,180 ,80 );
  connect( bg, SIGNAL(clicked(int)), SLOT(checkBoxClicked()) );
  if (save)
    cb[0] -> setChecked(TRUE);
  if (plot)
    cb[1] -> setChecked(TRUE);
}
void Options::checkBoxClicked() {
  if ( cb[0]->isChecked() ) {
    enable = 1;
  }
  else {
    enable = 0;
  }
  if ( cb[1]->isChecked() )
    plot = 1;
  else
    plot = 0;

  //printf("check selected en %d plo %d \n",enable,plot);
}
void Options::cancel_pressed() {
 delete this;
}
void Options::ok_pressed() {
  //  printf("ok pressed \n");
  emit(SIG_setoptions(enable,plot));
  delete this;
}


/*******************************************************************************
**  Options Widget : this is small widget to show one QLineEdit
**  to enter period for internal trigger
********************************************************************************/
//defines for options widget
//#define XSize 200
//#define YSize 150
DelayWidget::DelayWidget(QWidget *parent,const char* name)
  : QWidget(parent, name)
{
  label1    = new QLabel(this,"headline1");
  label1   -> setText("Enter period in ms : ");
  headline  = new QLabel(this,"headline");
  headline -> setFont(QFont("Helvetica",14));
  headline -> setText("Internal Trigger Period Editor :");
  DelayEdit  = new QLineEdit(this,"YScale");
  DelayEdit-> setMaxLength(10);
  ok        = new QPushButton(this,"ok");
  cancel    = new QPushButton(this,"cancel");
  ok       -> setText("o.k.");
  cancel   -> setText("cancel");
  connect(ok,SIGNAL(clicked()),SLOT(okpressed()));
  connect(cancel,SIGNAL(clicked()),SLOT(cancelpressed()));
  setFixedSize(XSize,YSize);//window has fixed size -> small
  headline  -> setGeometry(10,10,XSize,20);
  label1    -> setGeometry(50,30,XSize-40,20);
  DelayEdit -> setGeometry(60,50,75,20);
  ok        -> setGeometry(25,100,50,30);
  cancel    -> setGeometry(125,100,50,30);
}
void DelayWidget :: Setdelay(uint y)
{
  QString str;
  str.sprintf("%d",y);       //copy int to string
  DelayEdit -> setText(str); //setText in line Edit Window
}
void DelayWidget :: okpressed() {
  QString str;
  str = DelayEdit  -> text();
  if (str.toInt()>=0)
    emit(SIG_setdelay((uint) str.toInt() )); //call scheduler routine to set the value
  hide();
}
void DelayWidget :: cancelpressed() {
 hide();
}

/*******************************************************************************
**  Update Intervall Widget : this is small widget to show one QLineEdit
**  to enter updateIntervall for plotwindow
********************************************************************************/

UpdateWidget::UpdateWidget(QWidget *parent,const char* name)
  : QWidget(parent, name)
{
  setCaption("Timing Editor");	
  /*set buttons & text for max options */
  label1    = new QLabel(this,"headline1");
  label1   -> setText("- Units are ms -");
  headline  = new QLabel(this,"headline");
  //  headline -> setFont(QFont("Helvetica",14));
  headline -> setText("Plotwindow Update Period :");
  DelayEdit  = new QLineEdit(this,"YScale");
  DelayEdit-> setMaxLength(10);

  /*set buttons and text for polld options*/
  label2    = new QLabel(this,"headline2");
  label2    -> setText("Send Data Period : ");
  DelayEdit2 = new QLineEdit(this,"bla");
  DelayEdit2-> setMaxLength(10);

  /*now the rest*/
  ok        = new QPushButton(this,"ok");
  ok       -> setDefault(TRUE);
  cancel    = new QPushButton(this,"cancel");
  ok       -> setText("o.k.");
  cancel   -> setText("cancel");
  connect(ok,SIGNAL(clicked()),SLOT(okpressed()));
  connect(cancel,SIGNAL(clicked()),SLOT(cancelpressed()));
  setFixedSize(XSize,YSize+20);//window has fixed size -> small
  headline  -> setGeometry(30,30,XSize,20);
  label1    -> setGeometry(50,10,XSize-40,20);
  DelayEdit -> setGeometry(60,50,75,20);
  label2    -> setGeometry(50,70,XSize-40,20);
  DelayEdit2-> setGeometry(60,90,75,20);
  ok        -> setGeometry(25,120,50,30);
  cancel    -> setGeometry(125,120,50,30);
}
void UpdateWidget :: Setupdate(uint y,uint x)
{
  QString str;
  str.sprintf("%d",y);       //copy int to string
  DelayEdit -> setText(str); //setText in line Edit Window
  str.sprintf("%d",x);
  DelayEdit2 -> setText(str);
}
void UpdateWidget :: okpressed() {
  QString str,str2;
  str = DelayEdit  -> text();
  str2 =DelayEdit2 -> text();
  if (str.toInt()>=0)
    emit(SIG_setupdate((uint) str.toInt(),
		       (uint) str2.toInt() )); //call scheduler routine to set the value
  hide();
}
void UpdateWidget :: cancelpressed() {
 hide();
}
/*****************************************************************************
**  Save Widget : this is small widget to enter save filename
******************************************************************************/
SaveWidget :: SaveWidget(QWidget *parent, const char *name )
  : QWidget( parent, name )
{
  QBoxLayout *submenu  = new QHBoxLayout();
  QBoxLayout *submenu2 = new QHBoxLayout();

  tab = new QTabDialog( 0, "top-level dialog" );
  tab -> setCaption("Save Data and Info Files");

  tab -> setCancelButton();

  //add new widgets as elements of tabdialog pages
  QWidget *w1 = new QWidget(tab,"page-one");
  QWidget *w2 = new QWidget(tab,"page-two");



  // Now create first page of tab dialog

  QGridLayout *g1 = new QGridLayout(w1,10,3,2);  //3 rows ten colummns
//add submenues
  g1 -> addLayout(submenu,7,1);  //add index and format selector titles;
  g1 -> addLayout(submenu2,8,1);   //add index and format selector widgets (Qlineedit * QComboBox)

  headline = new QLabel(w1, "file_label");
  headline -> setText("Enter filenames : ");
  g1 -> addWidget(headline,0,0);

  //title  of label for last filename
  last_dataFname = new QLabel(w1,"lastf");
  last_dataFname -> setText("Last Data Filename :");
  g1 -> addWidget(last_dataFname,1,0);

  //text label for filename
  last_dFname = new QLabel(w1,"lastd");
  last_dFname -> setFont(QFont("Helvetica",11,QFont::Bold));
  g1 -> addWidget(last_dFname,1,1);

  editor = new QLineEdit(w1, "deveditor");
  g1 -> addWidget(editor,2,1);

  browse1 = new QPushButton(w1,"browse");
  browse1 -> setText("Browse..");
  g1 -> addWidget(browse1,2,2);

  title1  = new QLabel(w1, "file_label");
  title1 -> setText("Data Filename");
  g1 -> addWidget(title1,2,0);

  filename = new char [100];
  assert(filename != 0);

  editor2 = new QLineEdit(w1, "deveditor");
  g1 -> addWidget(editor2,4,1);

  browse2 = new QPushButton(w1,"browse");
  browse2 -> setText("Browse..");
  g1 -> addWidget(browse2,4,2);

  title2  = new QLabel(w1, "file_label");
  title2 -> setText("Array Filename");
  g1 -> addWidget(title2,4,0);

  last_arrayFname = new QLabel(w1,"lastf");
  last_arrayFname -> setText("Last Array Filename :");
  //  last_arrayFname -> setFont(QFont("Helvetica",10));
  g1 -> addWidget(last_arrayFname,3,0);

  last_aFname = new QLabel(w1,"lastaf");
  last_aFname ->  setFont(QFont("Helvetica",11,QFont::Bold));
  g1 -> addWidget(last_aFname,3,1);

  editor3 = new QLineEdit(w1, "deveditor");
  g1 -> addWidget(editor3,6,1);

  browse3 = new QPushButton(w1,"browse");
  browse3 -> setText("Browse..");
  g1 -> addWidget(browse3,6,2);

  title3  = new QLabel(w1, "file_label");
  title3 -> setText("Image Filename");
  g1 -> addWidget(title3,6,0);

  last_imFname = new QLabel(w1,"lastf");
  last_imFname -> setText("Last Image Filename :");
  g1 -> addWidget(last_imFname,5,0);

  last_iFname = new QLabel(w1,"imname");
  last_iFname -> setFont(QFont("Helvetica",11,QFont::Bold));
  g1 -> addWidget(last_iFname,5,1);

  editor4 = new QLineEdit(w1, "deveditor");
  g1 -> addWidget(editor4,9,1);

  browse4 = new QPushButton(w1,"browse");
  browse4 -> setText("Browse..");
  g1 -> addWidget(browse4,9,2);

  title4  = new QLabel(w1, "file_label");
  title4 -> setText("Working Directory");
  g1 -> addWidget(title4,9,0);

  title5  = new QLabel(w1, "index");
  title5 -> setText("index");
  submenu -> addWidget(title5,1,AlignCenter);

  editor5 = new QLineEdit(w1, "indexeditor");
  editor5 -> setText("0");
  submenu2 -> addWidget(editor5,1,AlignCenter);

  FormatSelector = new QComboBox(FALSE,w1, "format");
  FormatSelector -> insertItem("PBM");
  FormatSelector -> insertItem("BMP");
  FormatSelector -> insertItem("XPM");
  FormatSelector -> insertItem("PGM");
  FormatSelector -> insertItem("PPM");
  submenu2 -> addWidget(FormatSelector,1,AlignCenter);

  ComboTitle = new QLabel(w1,"Combo");
  ComboTitle -> setText("Image Format");
  submenu -> addWidget(ComboTitle,1,AlignCenter);

  connect(tab,SIGNAL(cancelButtonPressed()),SLOT(cancelpressed()));
  connect(tab,SIGNAL(applyButtonPressed()),SLOT(okpressed()));
  connect(browse1,SIGNAL(clicked()),SLOT(browseclicked()));
  connect(browse2,SIGNAL(clicked()),SLOT(browseclicked2()));
  connect(browse3,SIGNAL(clicked()),SLOT(browseclicked3()));
  connect(browse4,SIGNAL(clicked()),SLOT(browseclicked4()));

  // add first page of TabDialog and activate it
  tab -> addTab(w1,"file");
  g1 -> activate();
  //
  //create second page of TabDialog
  //
  //add Grid for layout for second page of Tab Dialog  -> widgets are arranged automatically
  //only one row and column but 10 dots spacing to the border
  //
  QGridLayout *g2 = new QGridLayout(w2,1,1,10);
  //
  // it only contains one multiLineEdit to edit the parameter file
  //
  ed1 = new QMultiLineEdit( w2 ); //child of second page w2
  g2  -> addWidget( ed1,0,0);     //Add multilineedit to widget;
  ed1 -> setText( " " );
  g2  -> setColStretch(0,0);      //let it stretch
  g2  -> setRowStretch(0,0);
  //
  //add second page to tab dialog
  tab->addTab( w2, "text" );
  //activate Layout manager for it
  g2 -> activate();

  array_filename = new char [100];
  assert(array_filename != 0);
  directory      = new char [100];
  assert(directory != 0);
  image_filename = new char [100];
  assert(image_filename != 0);
  l_dataname = new char[100];
  assert( l_dataname != 0);
  l_arrayname = new char[100];
  assert( l_arrayname != 0);
  l_imname = new char[100];
  assert( l_imname != 0);

  tab -> resize(SAVE_WIDGET_X_SIZE, SAVE_WIDGET_Y_SIZE);
  tab ->show();
}

void SaveWidget :: SetFilename(char *save_filename,char *a_filename,char *i_filename,
			       char *dir,char *ldata,char *larray,char *limage)
{
  strcpy(filename,save_filename);   //copy filename in lokal buffer
  editor->setText(save_filename);
  strcpy(array_filename,a_filename);//copy filename in lokal buffer
  editor2->setText(a_filename);
  strcpy(image_filename,i_filename);
  editor3->setText(i_filename);
  strcpy(directory,dir);
  editor4->setText(dir);
  last_aFname -> setText(larray);
  last_iFname -> setText(limage);
  last_dFname -> setText(ldata);
  return;
}
void SaveWidget :: SetText(QString text)
{
  ed1 -> setText(text);
}
//
//also change arguments of SIGNAL(SIG_Save)
//
void SaveWidget :: okpressed() {
  int index;
  index = atoi(editor5 -> text());
  strcpy(directory,editor4     ->text());
  strcpy(filename,editor4      ->text());
  strcpy(array_filename,editor4->text());
  strcpy(image_filename,editor4->text());
  strcat(filename,editor->text());         //scalar data
  strcat(array_filename,editor2 -> text());  //arrays
  strcat(image_filename,editor3 -> text());//image
  QString str,str2,str3;	
  QString namebuffer;
  namebuffer.sprintf("%s.dat",filename);
  QFileInfo fi(namebuffer); //check if file exists already
  str+= namebuffer;
  str.append(" exists\noverwrite it ? \n");
  int ok=1,ok2=1,ok3=1;
  if( fi.exists() ){
   switch ( QMessageBox::warning(0,"WARNING",str,
			   QMessageBox::No | QMessageBox::Default,
			   QMessageBox::Yes  | QMessageBox::Escape )) {
  case QMessageBox::Yes :
    ok =1;
    break;
  case QMessageBox::No :
    ok = 0;	
    break;
   }
  }
  if (ok) {  //only check further if first one was successful
    namebuffer.sprintf("%s.str",array_filename);	
    QFileInfo fa(namebuffer); //check if file exists already
    str2+= namebuffer;
    str2.append(" exists\noverwrite it ? \n");
    if( fa.exists() ){
      switch ( QMessageBox::warning(0,"WARNING",str2,	
				    QMessageBox::No | QMessageBox::Default,
				    QMessageBox::Yes  | QMessageBox::Escape )) {
      case QMessageBox::Yes :
	ok =1;
	break;
      case QMessageBox::No :
	ok = 0;	
	break;
      }
    }
  }
  if (ok && ok2) { //dito
    namebuffer.sprintf("%s.%s",array_filename,
		       (char *) FormatSelector -> currentText());

    QFileInfo fim(array_filename); //check if file exists already
    str3+= namebuffer;
    str3.append(" exists\noverwrite it ? \n");
    if( fim.exists() ){
      switch ( QMessageBox::warning(0,"WARNING",str3,
				    QMessageBox::No | QMessageBox::Default,
				    QMessageBox::Yes  | QMessageBox::Escape )) {
      case QMessageBox::Yes :
	ok =1;
	break;
      case QMessageBox::No :
	ok = 0;	
	break;
      }
    }
  }
  if (ok && ok2 && ok3 ) {
    emit(SIG_save(filename,array_filename,image_filename,
		  index, (char *) FormatSelector -> currentText(),
		  (const char *) ed1 -> text() , (const char *) editor4 -> text()
		  ) );
    delete this; // hide();
  }
}

void SaveWidget :: cancelpressed() {
  delete this;   //hide();
}
// Filedialog for scalar data filename
void SaveWidget :: browseclicked() {
  int pos;	
  QString f = QFileDialog::getSaveFileName( 0, "*.dat", 0 , "Data-File" );
    if ( !f.isEmpty() ) {
      f.length(); //now erase the path, just show names
      pos = f.findRev('/',f.length());
      editor -> setText(f.right(f.length()-pos-1));
    } else {
        // the user cancelled the dialog
    }
}

//Filedialog for srray filename
void SaveWidget :: browseclicked2() {
  int pos;
  QString f = QFileDialog::getSaveFileName( 0, "*.str", 0/*this*/ , "Array-File" );
  if ( !f.isEmpty() ) {
    f.length(); //now erase the path, just show names
    pos = f.findRev('/',f.length());
    editor2 -> setText(f.right(f.length()-pos-1));
  } else {
        // the user cancelled the dialog
    }
}
void SaveWidget :: browseclicked3() {
  int pos;
  char wc[6];
  sprintf(wc,"*.%s",(const char *) (FormatSelector -> currentText()));
  QString f = QFileDialog::getSaveFileName( 0,wc, 0/*this*/ , "Image-File" );
  if ( !f.isEmpty() ) {
    f.length(); //now erase the path, just show names
    pos = f.findRev('/',f.length());
    editor3 -> setText(f.right(f.length()-pos-1));
    } else {
        // the user cancelled the dialog
    }
}
void SaveWidget :: browseclicked4() {
  QString d = QFileDialog::getExistingDirectory();
  //  QString f = QFileDialog::getSaveFileName( 0,"./", this , "Image-File" );
    if ( !d.isEmpty() ) {
      editor4 -> setText(d);
    } else {
        // the user cancelled the dialog
    }
}
void SaveWidget :: PaintEvent(QPaintEvent *)
{
  tab->  repaint();
}
void SaveWidget :: ResizeEvent( QResizeEvent *) {
  tab->repaint();
}
